@@ -355,9 +355,9 @@ noteList BagpipeEmbellishment::getNoteList() const
 //   write
 //---------------------------------------------------------
 
-void BagpipeEmbellishment::write(Xml& xml) const
+void BagpipeEmbellishment::write(XmlWriter& xml) const
       {
-      xml.stag(name());
+      xml.stag(this);
       xml.tag("subtype", _embelType);
       xml.etag();
       }
@@ -458,7 +458,7 @@ static void symMetrics(const char* name, const Sym& headsym)
              headsym.getAttach().y());
       }
 */
-      
+
 //---------------------------------------------------------
 //   mag
 //      return fixed magnification
