@@ -63,12 +63,7 @@ struct TextSegment {
 struct RenderAction;
 class HDegree;
 
-class Harmony : public Text {
-      Q_OBJECT
-      Q_PROPERTY(int baseTpc  READ baseTpc  WRITE setBaseTpc)
-      Q_PROPERTY(int id  READ id  WRITE setId)
-      Q_PROPERTY(int rootTpc  READ rootTpc  WRITE setRootTpc)
-
+class Harmony final : public TextBase {
       int _rootTpc;                       // root note for chord
       int _baseTpc;                       // bass note or chord base; used for "slash" chords
                                           // or notation of base note in chord
@@ -78,6 +73,7 @@ class Harmony : public Text {
       QString _userName;                  // name as typed by user if applicable
       QString _textName;                  // name recognized from chord list, read from score file, or constructed from imported source
       ParsedChord* _parsedForm;           // parsed form of chord
+      bool showSpell = false;             // show spell check warning
 
       QList<HDegree> _degreeList;
       QList<QFont> fontList;              // temp values used in render()
@@ -93,21 +89,23 @@ class Harmony : public Text {
 
       void determineRootBaseSpelling();
       virtual void draw(QPainter*) const override;
+      virtual void drawEditMode(QPainter* p, EditData& ed) override;
       void render(const QString&, qreal&, qreal&);
       void render(const QList<RenderAction>& renderList, qreal&, qreal&, int tpc, NoteSpellingType noteSpelling = NoteSpellingType::STANDARD, NoteCaseType noteCase = NoteCaseType::AUTO);
-      virtual void styleChanged() override     { render(); }
-      virtual void setTextStyle(const TextStyle& st) override;
+      virtual Sid getPropertyStyle(Pid) const override;
 
    public:
       Harmony(Score* = 0);
       Harmony(const Harmony&);
       ~Harmony();
       virtual Harmony* clone() const override     { return new Harmony(*this); }
-      virtual Element::Type type() const override { return Element::Type::HARMONY; }
-      virtual bool systemFlag() const override    { return false;  }
+      virtual ElementType type() const override   { return ElementType::HARMONY; }
 
-      void setId(int d)                        { _id = d; }
-      int id() const                           { return _id;           }
+      void setId(int d)                        { _id = d;       }
+      int id() const                           { return _id;    }
+
+      void setBaseCase(NoteCaseType c)         { _baseCase = c; }
+      void setRootCase(NoteCaseType c)         { _rootCase = c; }
 
       bool leftParen() const                   { return _leftParen;    }
       bool rightParen() const                  { return _rightParen;   }
@@ -120,19 +118,17 @@ class Harmony : public Text {
       const ChordDescription* getDescription(const QString&, const ParsedChord* pc = 0);
       const ChordDescription* generateDescription();
 
-      void determineRootBaseSpelling(NoteSpellingType& rootSpelling, NoteCaseType& rootCase, NoteSpellingType& baseSpelling, NoteCaseType& baseCase);
+      void determineRootBaseSpelling(NoteSpellingType& rootSpelling, NoteCaseType& rootCase,
+         NoteSpellingType& baseSpelling, NoteCaseType& baseCase);
 
-      virtual void textChanged() override;
+      void textChanged();
       virtual void layout() override;
-
-      const QRectF& bboxtight() const          { return _tbbox;        }
-      QRectF& bboxtight()                      { return _tbbox;        }
-      void setbboxtight(const QRectF& r) const { _tbbox = r;           }
+      virtual void layout1() override;
 
       virtual bool isEditable() const override { return true; }
-      virtual void startEdit(MuseScoreView*, const QPointF&) override;
-      virtual bool edit(MuseScoreView*, Grip, int key, Qt::KeyboardModifiers, const QString& s) override;
-      virtual void endEdit() override;
+      virtual void startEdit(EditData&) override;
+      virtual bool edit(EditData&) override;
+      virtual void endEdit(EditData&) override;
 
       QString hUserName() const                { return _userName;     }
       QString hTextName() const                { return _textName;     }
@@ -150,24 +146,23 @@ class Harmony : public Text {
       const QList<HDegree>& degreeList() const;
       const ParsedChord* parsedForm();
 
-      virtual void write(Xml& xml) const override;
+      virtual void write(XmlWriter& xml) const override;
       virtual void read(XmlReader&) override;
-      QString harmonyName();
-      void render(const TextStyle* ts = 0);
+      QString harmonyName() const;
+      void render();
 
       const ChordDescription* parseHarmony(const QString& s, int* root, int* base, bool syntaxOnly = false);
 
       const QString& extensionName() const;
 
       QString xmlKind() const;
-      QString xmlText() const;
+      QString musicXmlText() const;
       QString xmlSymbols() const;
       QString xmlParens() const;
       QStringList xmlDegrees() const;
 
       void resolveDegreeList();
 
-//      virtual bool isEmpty() const override;
       virtual qreal baseLine() const override;
 
       const ChordDescription* fromXml(const QString&, const QString&, const QString&, const QString&, const QList<HDegree>&);
@@ -175,20 +170,20 @@ class Harmony : public Text {
       const ChordDescription* fromXml(const QString& s);
       virtual void spatiumChanged(qreal oldValue, qreal newValue) override;
       virtual void localSpatiumChanged(qreal oldValue, qreal newValue) override;
-      virtual void textStyleChanged() override;
       void setHarmony(const QString& s);
-      virtual QPainterPath shape() const override;
       void calculateBoundingRect();
 
-      virtual QString accessibleInfo() override;
-      virtual QString screenReaderInfo() override;
+      virtual QString accessibleInfo() const override;
+      virtual QString screenReaderInfo() const override;
 
-      virtual bool acceptDrop(const DropData&) const override;
-      virtual Element* drop(const DropData&) override;
+      virtual bool acceptDrop(EditData&) const override;
+      virtual Element* drop(EditData&) override;
 
+      virtual QVariant getProperty(Pid propertyId) const override;
+      virtual bool setProperty(Pid propertyId, const QVariant& v) override;
+      virtual QVariant propertyDefault(Pid id) const override;
       };
 
-
 }     // namespace Ms
 #endif
 
