@@ -92,8 +92,8 @@ void InstrumentGroup::read(XmlReader& e)
       while (e.readNextStartElement()) {
             const QStringRef& tag(e.name());
             if (tag == "instrument" || tag == "Instrument") {
-                  QString id = e.attribute("id");
-                  InstrumentTemplate* t = searchTemplate(id);
+                  QString sid = e.attribute("id");
+                  InstrumentTemplate* t = searchTemplate(sid);
                   if (t == 0) {
                         t = new InstrumentTemplate;
                         t->articulation.append(articulation);     // init with global articulation
@@ -147,6 +147,7 @@ InstrumentTemplate::InstrumentTemplate()
       useDrumset         = false;
       drumset            = 0;
       extended           = false;
+      singleNoteDynamics = true;
 
       for (int i = 0; i < MAX_STAVES; ++i) {
             clefTypes[i]._concertClef = ClefType::G;
@@ -155,7 +156,7 @@ InstrumentTemplate::InstrumentTemplate()
             smallStaff[i]  = false;
             bracket[i]     = BracketType::NO_BRACKET;
             bracketSpan[i] = 0;
-            barlineSpan[i] = 0;
+            barlineSpan[i] = false;
             }
       transpose.diatonic   = 0;
       transpose.chromatic  = 0;
@@ -201,6 +202,7 @@ void InstrumentTemplate::init(const InstrumentTemplate& t)
       stringData  = t.stringData;
       midiActions = t.midiActions;
       channel     = t.channel;
+      singleNoteDynamics = t.singleNoteDynamics;
       }
 
 InstrumentTemplate::~InstrumentTemplate()
@@ -212,7 +214,7 @@ InstrumentTemplate::~InstrumentTemplate()
 //   write
 //---------------------------------------------------------
 
-void InstrumentTemplate::write(Xml& xml) const
+void InstrumentTemplate::write(XmlWriter& xml) const
       {
       xml.stag(QString("Instrument id=\"%1\"").arg(id));
       longNames.write(xml, "longName");
@@ -272,7 +274,7 @@ void InstrumentTemplate::write(Xml& xml) const
                   else
                         xml.tag("bracketSpan", bracketSpan[i]);
                   }
-            if (barlineSpan[i] != 0) {
+            if (barlineSpan[i]) {
                   if (i)
                         xml.tag(QString("barlineSpan staff=\"%1\"").arg(i+1), barlineSpan[i]);
                   else
@@ -291,13 +293,17 @@ void InstrumentTemplate::write(Xml& xml) const
             xml.tag("drumset", int(useDrumset));
       if (drumset)
             drumset->save(xml);
-      foreach(const NamedEventList& a, midiActions)
+      
+      if (!singleNoteDynamics)      // default is true
+            xml.tag("singleNoteDynamics", singleNoteDynamics);
+
+      for (const NamedEventList& a : midiActions)
             a.write(xml, "MidiAction");
-      foreach(const Channel& a, channel)
-            a.write(xml);
-      foreach(const MidiArticulation& ma, articulation) {
+      for (const Channel& a : channel)
+            a.write(xml, nullptr);
+      for (const MidiArticulation& ma : articulation) {
             bool isGlobal = false;
-            foreach(const MidiArticulation& ga, Ms::articulation) {
+            for (const MidiArticulation& ga : Ms::articulation) {
                   if (ma == ga) {
                         isGlobal = true;
                         break;
@@ -314,7 +320,7 @@ void InstrumentTemplate::write(Xml& xml) const
 //    output only translatable names
 //---------------------------------------------------------
 
-void InstrumentTemplate::write1(Xml& xml) const
+void InstrumentTemplate::write1(XmlWriter& xml) const
       {
       xml.stag(QString("Instrument id=\"%1\"").arg(id));
       longNames.write(xml, "longName");
@@ -332,25 +338,27 @@ void InstrumentTemplate::write1(Xml& xml) const
 void InstrumentTemplate::read(XmlReader& e)
       {
       id = e.attribute("id");
-      int drumsetIndex = 0;
+
       while (e.readNextStartElement()) {
             const QStringRef& tag(e.name());
 
             if (tag == "longName" || tag == "name") {               // "name" is obsolete
                   int pos = e.intAttribute("pos", 0);
                   for (QList<StaffName>::iterator i = longNames.begin(); i != longNames.end(); ++i) {
-                        if ((*i).pos() == pos)
+                        if ((*i).pos() == pos) {
                               longNames.erase(i);
-                        break;
+                              break;
+                              }
                         }
                   longNames.append(StaffName(qApp->translate("InstrumentsXML", e.readElementText().toUtf8().data()), pos));
                   }
             else if (tag == "shortName" || tag == "short-name") {   // "short-name" is obsolete
                   int pos = e.intAttribute("pos", 0);
                   for (QList<StaffName>::iterator i = shortNames.begin(); i != shortNames.end(); ++i) {
-                        if ((*i).pos() == pos)
+                        if ((*i).pos() == pos) {
                               shortNames.erase(i);
-                        break;
+                              break;
+							  }
                         }
                   shortNames.append(StaffName(qApp->translate("InstrumentsXML", e.readElementText().toUtf8().data()), pos));
                   }
@@ -363,7 +371,8 @@ void InstrumentTemplate::read(XmlReader& e)
             else if (tag == "staves") {
                   staves = e.readInt();
                   bracketSpan[0] = staves;
-                  barlineSpan[0] = staves;
+//                  for (int i = 0; i < staves-1; ++i)
+//                        barlineSpan[i] = true;
                   }
             else if (tag == "clef") {           // sets both transposing and concert clef
                   int idx = readStaffIdx(e);
@@ -406,7 +415,9 @@ void InstrumentTemplate::read(XmlReader& e)
                   }
             else if (tag == "barlineSpan") {
                   int idx = readStaffIdx(e);
-                  barlineSpan[idx] = e.readInt();
+                  int span = e.readInt();
+                  for (int i = 0; i < span-1; ++i)
+                        barlineSpan[idx+i] = true;
                   }
             else if (tag == "aPitchRange")
                   setPitchRange(e.readElementText(), &minPitchA, &maxPitchA);
@@ -432,8 +443,7 @@ void InstrumentTemplate::read(XmlReader& e)
                         drumset = new Drumset(*smDrumset);
                         drumset->clear();
                         }
-                  drumset->load(e, drumsetIndex);
-                  drumsetIndex++;
+                  drumset->load(e);
                   }
             else if (tag == "MidiAction") {
                   NamedEventList a;
@@ -442,7 +452,7 @@ void InstrumentTemplate::read(XmlReader& e)
                   }
             else if (tag == "Channel" || tag == "channel") {
                   Channel a;
-                  a.read(e);
+                  a.read(e, nullptr);
                   channel.append(a);
                   }
             else if (tag == "Articulation") {
@@ -492,43 +502,25 @@ void InstrumentTemplate::read(XmlReader& e)
                   QString val(e.readElementText());
                   linkGenre(val);
                   }
+            else if (tag == "singleNoteDynamics")
+                  singleNoteDynamics = e.readBool();
             else
                   e.unknown();
             }
-      //
-      // check bar line spans
-      //
-      int barLine = 0;
-      for (int i = 0; i < staves; ++i) {
-            int bls = barlineSpan[i];
-            if (barLine) {
-                  if (bls)
-                        barlineSpan[i] = 0;
-                  }
-            else {
-                  if (bls == 0) {
-                        bls = 1;
-                        barlineSpan[i] = 1;
-                        }
-                  barLine = bls;
-                  }
-            --barLine;
-            }
-      if (channel.isEmpty()) {
+      if (channel.empty()) {
             Channel a;
-            a.chorus       = 0;
-            a.reverb       = 0;
-            a.name         = "normal";
-            a.program      = 0;
-            a.bank         = 0;
-            a.volume       = 100;
-            a.pan          = 64; // actually 63.5 for center
+            a.setChorus(0);
+            a.setReverb(0);
+            a.setName("normal");
+            a.setProgram(0);
+            a.setBank(0);
+            a.setVolume(90);
+            a.setPan(0);
             channel.append(a);
             }
       if (useDrumset) {
-            if (channel[0].bank == 0 && channel[0].name != "Zerberus")
-                  channel[0].bank = 128;
-            channel[0].updateInitList();
+            if (channel[0].bank() == 0 && channel[0].synti().toLower() != "zerberus")
+                  channel[0].setBank(128);
             }
       if (trackName.isEmpty() && !longNames.isEmpty())
             trackName = longNames[0].name();
@@ -569,7 +561,7 @@ bool saveInstrumentTemplates(const QString& instrTemplates)
             qDebug("cannot save instrument templates at <%s>", qPrintable(instrTemplates));
             return false;
             }
-      Xml xml(&qf);
+      XmlWriter xml(0, &qf);
       xml << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
       xml.stag("museScore");
       foreach(const InstrumentGenre* genre, instrumentGenres)
@@ -583,7 +575,7 @@ bool saveInstrumentTemplates(const QString& instrTemplates)
             xml.tag("name", group->name);
             if (group->extended)
                   xml.tag("extended", group->extended);
-            foreach(InstrumentTemplate* it, group->instrumentTemplates) {
+            for (InstrumentTemplate* it : group->instrumentTemplates) {
                   it->write(xml);
                   xml << "\n";
                   }
@@ -606,7 +598,7 @@ bool saveInstrumentTemplates1(const QString& instrTemplates)
             qDebug("cannot save instrument templates at <%s>", qPrintable(instrTemplates));
             return false;
             }
-      Xml xml(&qf);
+      XmlWriter xml(0, &qf);
       xml << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
       xml.stag("museScore");
       foreach(const InstrumentGenre* genre, instrumentGenres)
@@ -614,7 +606,7 @@ bool saveInstrumentTemplates1(const QString& instrTemplates)
       foreach(InstrumentGroup* group, instrumentGroups) {
             xml.stag(QString("InstrumentGroup id=\"%1\"").arg(group->id));
             xml.tag("name", group->name);
-            foreach(InstrumentTemplate* it, group->instrumentTemplates) {
+            for (InstrumentTemplate* it : group->instrumentTemplates) {
                   it->write1(xml);
                   xml << "\n";
                   }
@@ -698,8 +690,8 @@ bool loadInstrumentTemplates(const QString& instrTemplates)
 
 InstrumentTemplate* searchTemplate(const QString& name)
       {
-      foreach(InstrumentGroup* g, instrumentGroups) {
-            foreach(InstrumentTemplate* it, g->instrumentTemplates) {
+      for (InstrumentGroup* g : instrumentGroups) {
+            for (InstrumentTemplate* it : g->instrumentTemplates) {
                   if (it->id == name)
                         return it;
                   }
@@ -713,8 +705,8 @@ InstrumentTemplate* searchTemplate(const QString& name)
 
 InstrumentTemplate* searchTemplateForMusicXmlId(const QString& mxmlId)
       {
-      foreach(InstrumentGroup* g, instrumentGroups) {
-            foreach(InstrumentTemplate* it, g->instrumentTemplates) {
+      for (InstrumentGroup* g : instrumentGroups) {
+            for (InstrumentTemplate* it : g->instrumentTemplates) {
                   if (it->musicXMLid == mxmlId)
                         return it;
                   }
@@ -753,14 +745,14 @@ bool InstrumentTemplate::genreMember(const QString& name)
             return rVal;
       }
 
-void InstrumentGenre::write(Xml& xml) const
+void InstrumentGenre::write(XmlWriter& xml) const
       {
       xml.stag(QString("Genre id=\"%1\"").arg(id));
       xml.tag("name", name);
       xml.etag();
       }
 
-void InstrumentGenre::write1(Xml& xml) const
+void InstrumentGenre::write1(XmlWriter& xml) const
       {
       write(xml);
       }
@@ -789,6 +781,30 @@ ClefTypeList InstrumentTemplate::clefType(int staffIdx) const
       return clefTypes[0];
       }
 
+//---------------------------------------------------------
+//   defaultClef
+//    traverse the instrument list for first instrument
+//    with midi patch 'program'. Return the default clef
+//    for this instrument.
+//---------------------------------------------------------
+
+ClefType defaultClef(int program)
+      {
+      if (program >= 25 && program < 32)              // this are guitars
+            return ClefType::G8_VB;
+      else if (program >= 33 && program < 41)         // this is bass
+            return ClefType::F8_VB;
+
+      for (InstrumentGroup* g : instrumentGroups) {
+            for (InstrumentTemplate* it : g->instrumentTemplates) {
+                  if (it->channel[0].bank() == 0 && it->channel[0].program() == program){
+                        return (it->clefTypes[0]._concertClef);
+                        }
+                  }
+            }
+      return ClefType::G;
+      }
+
 }
 
 
