@@ -42,10 +42,9 @@ Measure* Score::searchLabel(const QString& s, Measure* startMeasure, Measure* en
       endMeasure = endMeasure->nextMeasure(); // stop comparison needs measure one past the last one to check
       for (Measure* m = startMeasure; m && (m != endMeasure); m = m->nextMeasure()) {
             for (auto e : m->el()) {
-                  if (e->type() == Element::Type::MARKER) {
-                        const Marker* marker = static_cast<const Marker*>(e);
-                        if (marker->label() == s)
-                              return m;
+                  if (   (e->isMarker())
+                      && (toMarker(e)->label() == s)) {
+                        return m;
                         }
                   }
             }
@@ -100,14 +99,15 @@ RepeatSegment::RepeatSegment(RepeatSegment * const rs, Measure * const fromMeasu
             ++it;
             }
       if (!measureList.empty()) {
-            tick = measureList.cbegin()->first->tick();
+            tick = measureList.cbegin()->first->tick().ticks();
             }
       }
 
 void RepeatSegment::addMeasure(Measure * const m)
       {
+      Q_ASSERT(measureList.empty() || measureList.back().first->nextMeasure() == m);
       if (measureList.empty()) {
-            tick = m->tick();
+            tick = m->tick().ticks();
             }
       measureList.push_back(std::make_pair(m, m->playbackCount()));
       }
@@ -124,7 +124,7 @@ bool RepeatSegment::containsMeasure(Measure const * const m) const
 
 int RepeatSegment::len() const
       {
-      return (measureList.empty()) ? 0 : (measureList.last().first->endTick() - tick);
+      return (measureList.empty()) ? 0 : (measureList.last().first->endTick().ticks() - tick);
       }
 
 //---------------------------------------------------------
@@ -154,14 +154,23 @@ RepeatList::RepeatList(Score* s)
       idx2  = 0;
       }
 
+//---------------------------------------------------------
+//   ~RepeatList
+//---------------------------------------------------------
+
+RepeatList::~RepeatList()
+      {
+      qDeleteAll(*this);
+      }
+
 //---------------------------------------------------------
 //   ticks
 //---------------------------------------------------------
 
-int RepeatList::ticks()
+int RepeatList::ticks() const
       {
       if (length() > 0) {
-            RepeatSegment* s = last();
+            const RepeatSegment* s = last();
             return s->utick + s->len();
             }
       return 0;
@@ -171,7 +180,24 @@ int RepeatList::ticks()
 //   update
 //---------------------------------------------------------
 
-void RepeatList::update()
+void RepeatList::update(bool expand)
+      {
+      if (!_scoreChanged && expand == _expanded)
+            return;
+
+      if (expand)
+            unwind();
+      else
+            flatten();
+
+      _scoreChanged = false;
+      }
+
+//---------------------------------------------------------
+//   updateTempo
+//---------------------------------------------------------
+
+void RepeatList::updateTempo()
       {
       const TempoMap* tl = _score->tempomap();
 
@@ -218,6 +244,8 @@ int RepeatList::utick2tick(int tick) const
 
 int RepeatList::tick2utick(int tick) const
       {
+      if (empty())
+            return 0;
       for (const RepeatSegment* s : *this) {
             if (tick >= s->tick && tick < (s->tick + s->len() ))
                   return s->utick + (tick - s->tick);
@@ -271,19 +299,48 @@ void RepeatList::dump() const
       {
 #if 0
       qDebug("==Dump Repeat List:==");
-      foreach(const RepeatSegment* s, *this) {
+      for (const RepeatSegment* s : *this) {
             qDebug("%p  tick: %3d(%d) %3d(%d) len %d(%d) beats  %f + %f", s,
                s->utick / MScore::division,
                s->utick / MScore::division / 4,
                s->tick / MScore::division,
                s->tick / MScore::division / 4,
-               s->len / MScore::division,
-               s->len / MScore::division / 4,
+               s->len() / MScore::division,
+               s->len() / MScore::division / 4,
                s->utime, s->timeOffset);
             }
 #endif
       }
 
+//---------------------------------------------------------
+//   flatten
+///   Make this repeat list flat (don't expand repeats)
+//---------------------------------------------------------
+
+void RepeatList::flatten()
+      {
+      qDeleteAll(*this);
+      clear();
+
+      Measure* m = _score->firstMeasure();
+      if (!m)
+            return;
+
+      RepeatSegment* s = new RepeatSegment;
+      s->tick  = 0;
+      s->utick = 0;
+      s->utime = 0.0;
+      s->timeOffset = 0.0;
+      do {
+            s->addMeasure(m);
+            m = m->nextMeasure();
+            }
+      while (m);
+      push_back(s);
+
+      _expanded = false;
+      }
+
 //---------------------------------------------------------
 //   unwind
 //    implements:
@@ -314,8 +371,8 @@ void RepeatList::unwind()
       MeasureBase* sectionStartMeasureBase = NULL; // NULL indicates haven't discovered starting Measure of section
       MeasureBase* sectionEndMeasureBase = NULL;
 
-      // partition score by section breaks and unwind individual sections seperately
-      // note: section breaks may occur on non-Measure frames, so must seach list of all MeasureBases
+      // partition score by section breaks and unwind individual sections separately
+      // note: section breaks may occur on non-Measure frames, so must search list of all MeasureBases
       for (MeasureBase* mb = _score->first(); mb; mb = mb->next()) {
 
             // unwindSection only deals with real Measures, so sectionEndMeasureBase and sectionStartMeasureBase will only point to real Measures
@@ -325,7 +382,7 @@ void RepeatList::unwind()
                   // starting measure of section will be the first non-NULL actual Measure encountered
                   if (sectionStartMeasureBase == NULL)
                         sectionStartMeasureBase = mb;
-            }
+                  }
 
             // if found section break or reached final MeasureBase of score, then unwind
             if (mb->sectionBreak() || !mb->nextMeasure()) {
@@ -333,7 +390,8 @@ void RepeatList::unwind()
                   // only unwind if section starts and ends with actual real measure
                   if (sectionStartMeasureBase && sectionEndMeasureBase) {
                         unwindSection(reinterpret_cast<Measure*>(sectionStartMeasureBase), reinterpret_cast<Measure*>(sectionEndMeasureBase));
-                        sectionStartMeasureBase = sectionEndMeasureBase = NULL; // reset to NULL to indicate that don't know starting Measure of next section after starting new section
+                        sectionStartMeasureBase = 0; // reset to NULL to indicate that don't know starting Measure of next section after starting new section
+                        sectionEndMeasureBase   = 0;
                         }
                   else {
                         qDebug( "Will not unroll a section that doesn't start or end with an actual measure. sectionStartMeasureBase = %p, sectionEndMeasureBase = %p",
@@ -342,7 +400,8 @@ void RepeatList::unwind()
                   }
             }
 
-      update();
+      updateTempo();
+      _expanded = true;
       dump();
       }
 
@@ -359,15 +418,15 @@ void RepeatList::preProcessVoltas()
       for (auto rit = _score->spanner().crbegin(); rit != _score->spanner().crend(); ++rit)
             {
             Spanner* s = (*rit).second;
-            if (s->type() != Element::Type::VOLTA)
+            if (!s->isVolta())
                   continue;
-            Volta* volta = static_cast<Volta*>(s);
+            Volta* volta = toVolta(s);
             if (volta) {
                   if (_voltaRanges.find(volta) == _voltaRanges.end()) { // not yet determined the real endpoint, should always be true
                         // start by assuming the end of the spanner == the end of this volta (closed volta)
                         Measure* voltaEndMeasure = volta->endMeasure();
                         // open volta may end past its spanner
-                        if (!volta->endHook()) {
+                        if (volta->getProperty(Pid::END_HOOK_TYPE).value<HookType>() == HookType::NONE) {
                               Measure* nextMeasureToInspect = voltaEndMeasure->nextMeasure();
                               // open volta ends:
                               while (  (nextMeasureToInspect)     // end of score
@@ -413,18 +472,18 @@ std::map<Volta*, Measure*>::const_iterator RepeatList::searchVolta(Measure * con
 //---------------------------------------------------------
 //   unwindSection
 //    unwinds from sectionStartMeasure through sectionEndMeasure
-//    appends repeat segments using rs
 //---------------------------------------------------------
 
 void RepeatList::unwindSection(Measure* const sectionStartMeasure, Measure* const sectionEndMeasure)
       {
 //      qDebug("unwind %d-measure section starting %p through %p", sectionEndMeasure->no()+1, sectionStartMeasure, sectionEndMeasure);
+
       if (!sectionStartMeasure || !sectionEndMeasure) {
             qDebug("invalid section start/end");
             return;
             }
 
-      rs = nullptr; // no measures to be played yet
+      RepeatSegment* rs = nullptr; // no measures to be played yet
 
       Measure* prevMeasure = nullptr; // the last processed measure that is part of this RepeatSegment
       Measure* currentMeasure = sectionStartMeasure; // the measure to be processed/evaluated
@@ -503,8 +562,8 @@ void RepeatList::unwindSection(Measure* const sectionStartMeasure, Measure* cons
                         // fetch the jump
                         Jump* jump = nullptr;
                         for (Element* e : currentMeasure->el()) {
-                              if (e->type() == Element::Type::JUMP) {
-                                    jump = static_cast<Jump*>(e);
+                              if (e->isJump()) {
+                                    jump = toJump(e);
                                     break;
                                     }
                               }
@@ -615,7 +674,7 @@ void RepeatList::unwindSection(Measure* const sectionStartMeasure, Measure* cons
                                                                   // this is the most recent playthrough of this measure -> copy it
                                                                   if (nullptr == rs) {
                                                                         rs = new RepeatSegment();
-                                                                        rs->tick = referenceIt->first->tick();
+                                                                        rs->tick = referenceIt->first->tick().ticks();
                                                                         }
                                                                   rs->measureList.push_back(std::make_pair(referenceIt->first, referenceIt->second));
                                                                   }
