@@ -2,7 +2,7 @@
 //  MuseScore
 //  Music Composition & Notation
 //
-//  Copyright (C) 2010-2017 Werner Schweer  & others
+//  Copyright (C) 2010-2015 Werner Schweer & others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2
@@ -23,7 +23,7 @@ namespace Ms {
 class Chord;
 class ChordRest;
 class Staff;
-class Xml;
+class XmlWriter;
 
 // all in spatium units
 #define STAFFTYPE_TAB_DEFAULTSTEMLEN_UP   3.0
@@ -37,7 +37,7 @@ class Xml;
 #define STAFFTYPE_TAB_DEFAULTDOTDIST_X    0.75
 
 // TAB STEM NOTATION
-// the ratio between the length of a full stem and the lenght of a short stem
+// the ratio between the length of a full stem and the length of a short stem
 // (used for half note stems, in some TAB styles)
 #define STAFFTYPE_TAB_SHORTSTEMRATIO      0.5
 // metrics of slashes through half note stems
@@ -78,7 +78,7 @@ class Xml;
 //   TablatureFont
 //---------------------------------------------------------
 
-#define NUM_OF_DIGITFRETS                 25    // the max fret number which can be rendered with numbers
+#define NUM_OF_DIGITFRETS                 100   // the max fret number which can be rendered with numbers
 #define NUM_OF_LETTERFRETS                17    // the max fret number which can be rendered with letters
 #define NUM_OF_BASSSTRING_SLASHES         5     // the max number of slashes supported for French bass strings notation
                                                 // (currently, only 3 slashes are used at most; another two are
@@ -126,7 +126,7 @@ enum class TablatureMinimStyle : char {
 
 enum class TablatureSymbolRepeat : char {
       NEVER = 0,                    // never repeat the same duration symbol
-      SYSTEM,                       // repeat at the begining of a new system
+      SYSTEM,                       // repeat at the beginning of a new system
       MEASURE,                      // repeat at the beginning of a new measure
       ALWAYS                        // always repeat
       };
@@ -175,19 +175,26 @@ class StaffType {
 
       StaffGroup _group = StaffGroup::STANDARD;
 
-      QString _xmlName;                   // the name used to reference this preset in intruments.xml
+      QString _xmlName;                   // the name used to reference this preset in instruments.xml
       QString _name;                      // user visible name
 
+      qreal _userMag           { 1.0   };       // allowed 0.1 - 10.0
+      Spatium _yoffset         { 0.0   };
+      bool _small              { false };
       int _lines            = 5;
       int _stepOffset       = 0;
       Spatium _lineDistance = Spatium(1);
 
-      bool _genClef         = true;       // create clef at beginning of system
       bool _showBarlines    = true;
+      bool _showLedgerLines = true;
       bool _slashStyle      = false;      // do not show stems
+
+      bool _genClef         = true;       // create clef at beginning of system
       bool _genTimesig      = true;       // whether time signature is shown or not
       bool _genKeysig       = true;       // create key signature at beginning of system
-      bool _showLedgerLines = true;
+
+      // Standard: configurable properties
+      NoteHeadScheme _noteHeadScheme = NoteHeadScheme::HEAD_NORMAL;
 
       // TAB: configurable properties
       qreal _durationFontSize = 15.0;     // the size (in points) for the duration symbol font
@@ -205,44 +212,43 @@ class StaffType {
       bool  _stemsDown    = true;         // stems are drawn downward (stem-and-beam durations only)
       bool  _stemsThrough = true;         // stems are drawn through the staff rather than beside it (stem-and-beam durations only)
       bool  _upsideDown   = false;        // whether lines are drawn with highest string at top (false) or at bottom (true)
-      bool  _showTabFingering = false;    // Allow fingering in tablature staff (true) or not (false)
+      bool  _showTabFingering   = false;        // Allow fingering in tablature staff (true) or not (false)
       bool  _useNumbers   = true;         // true: use numbers ('0' - ...) for frets | false: use letters ('a' - ...)
       bool  _showBackTied = true;         // whether back-tied notes are shown or not
 
       // TAB: internally managed variables
       // Note: values in RASTER UNITS are independent from score scaling and
       //    must be multiplied by magS() to be used in contexts using sp units
-      qreal _durationBoxH = 0.0;
-      qreal _durationBoxY = 0.0;          // the height and the y rect.coord. (relative to staff top line)
+      mutable qreal _durationBoxH = 0.0;
+      qreal mutable _durationBoxY = 0.0;          // the height and the y rect.coord. (relative to staff top line)
                                           // of a box bounding all duration symbols (raster units) internally computed:
                                           // depends upon _onString and the metrics of the duration font
-
       QFont _durationFont;                // font used to draw dur. symbols; cached for efficiency
       int   _durationFontIdx = 0;         // the index of current dur. font in dur. font array
-      qreal _durationYOffset = 0.0;       // the vertical offset to draw duration symbols with respect to the
+      mutable qreal _durationYOffset = 0.0;       // the vertical offset to draw duration symbols with respect to the
                                           // string lines (raster units); internally computed: depends upon _onString and duration font
-      qreal _durationGridYOffset = 0.0;   // the vertical offset to draw the bottom of duration grid with respect to the
+      mutable qreal _durationGridYOffset = 0.0;   // the vertical offset to draw the bottom of duration grid with respect to the
                                           // string lines (raster units); internally computed: depends upon _onstring and duration font
-      bool  _durationMetricsValid = false;  // whether duration font metrics are valid or not
-      qreal _fretBoxH = 0.0;
-      qreal _fretBoxY = 0.0;              // the height and the y rect.coord. (relative to staff line)
+      mutable bool  _durationMetricsValid = false;  // whether duration font metrics are valid or not
+      mutable qreal _fretBoxH = 0.0;
+      mutable qreal _fretBoxY = 0.0;              // the height and the y rect.coord. (relative to staff line)
                                           // of a box bounding all fret characters (raster units) internally computed:
                                           // depends upon _onString, _useNumbers and the metrics of the fret font
       QFont _fretFont;                    // font used to draw fret marks; cached for efficiency
       int   _fretFontIdx = 0;             // the index of current fret font in fret font array
-      qreal _fretYOffset = 0.0;           // the vertical offset to draw fret marks with respect to the string lines;
+      mutable qreal _fretYOffset = 0.0;           // the vertical offset to draw fret marks with respect to the string lines;
                                           // (raster units); internally computed: depends upon _onString, _useNumbers
                                           // and the metrics of the fret font
-      bool  _fretMetricsValid = false;    // whether fret font metrics are valid or not
-      qreal _refDPI = 0.0;                // reference value used to last computed metrics and to see if they are still valid
+      mutable  bool _fretMetricsValid = false;    // whether fret font metrics are valid or not
+      mutable qreal _refDPI = 0.0;                // reference value used to last computed metrics and to see if they are still valid
 
       // the array of configured fonts
       static QList<TablatureFretFont> _fretFonts;
       static QList<TablatureDurationFont> _durationFonts;
       static std::vector<StaffType> _presets;
 
-      void  setDurationMetrics();
-      void  setFretMetrics();
+      void  setDurationMetrics() const;
+      void  setFretMetrics() const;
 
       static bool readConfigFile(const QString& fileName);
       static const char    groupNames[STAFF_GROUP_MAX][STAFF_GROUP_NAME_MAX_LENGTH];      // used in UI
@@ -250,15 +256,16 @@ class StaffType {
 
    public:
       StaffType();
-      StaffType(StaffGroup sg, const QString& xml, const QString& name, int lines, qreal lineDist, bool genClef,
-            bool showBarLines, bool stemless, bool genTimeSig, bool genKeySig, bool showLedgerLines);
+      StaffType(StaffGroup sg, const QString& xml, const QString& name, int lines, int stpOff, qreal lineDist,
+            bool genClef, bool showBarLines, bool stemless, bool genTimeSig,
+            bool genKeySig, bool showLedgerLines);
 
-      StaffType(StaffGroup sg, const QString& xml, const QString& name, int lines, qreal lineDist, bool genClef,
-                  bool showBarLines, bool stemless, bool genTimesig,
-                  const QString& durFontName, qreal durFontSize, qreal durFontUserY, qreal genDur,
-                  const QString& fretFontName, qreal fretFontSize, qreal fretFontUserY, TablatureSymbolRepeat symRepeat,
-                  bool linesThrough, TablatureMinimStyle minimStyle, bool onLines, bool showRests,
-                  bool stemsDown, bool stemThrough, bool upsideDown, bool showTabFingering, bool useNumbers, bool showBackTied);
+      StaffType(StaffGroup sg, const QString& xml, const QString& name, int lines, int stpOff, qreal lineDist,
+            bool genClef, bool showBarLines, bool stemless, bool genTimesig,
+            const QString& durFontName, qreal durFontSize, qreal durFontUserY, qreal genDur,
+            const QString& fretFontName, qreal fretFontSize, qreal fretFontUserY, TablatureSymbolRepeat symRepeat,
+            bool linesThrough, TablatureMinimStyle minimStyle, bool onLines, bool showRests,
+            bool stemsDown, bool stemThrough, bool upsideDown, bool showTabFingering, bool useNumbers, bool showBackTied);
 
       virtual ~StaffType() {}
       bool operator==(const StaffType&) const;
@@ -272,7 +279,7 @@ class StaffType {
       const char* groupName() const;
       static const char* groupName(StaffGroup);
 
-      void setLines(int val);
+      void setLines(int val)                   { _lines = val;            }
       int lines() const                        { return _lines;           }
       void setStepOffset(int v)                { _stepOffset = v;         }
       int stepOffset() const                   { return _stepOffset;      }
@@ -282,8 +289,15 @@ class StaffType {
       bool genClef() const                     { return _genClef;         }
       void setShowBarlines(bool val)           { _showBarlines = val;     }
       bool showBarlines() const                { return _showBarlines;    }
+      qreal userMag() const                    { return _userMag;         }
+      bool small() const                       { return _small;           }
+      void setUserMag(qreal val)               { _userMag = val;          }
+      void setSmall(bool val)                  { _small = val;            }
+      Spatium yoffset() const                  { return _yoffset;         }
+      void setYoffset(Spatium val)             { _yoffset = val;          }
+      qreal spatium(Score*) const;
 
-      void write(Xml& xml) const;
+      void write(XmlWriter& xml) const;
       void read(XmlReader&);
 
       void setSlashStyle(bool val)             { _slashStyle = val;       }
@@ -302,11 +316,13 @@ class StaffType {
       bool genKeysig() const                   { return _genKeysig;         }
       void setShowLedgerLines(bool val)        { _showLedgerLines = val;    }
       bool showLedgerLines() const             { return _showLedgerLines;   }
+      void setNoteHeadScheme(NoteHeadScheme s) { _noteHeadScheme = s;       }
+      NoteHeadScheme noteHeadScheme() const    { return _noteHeadScheme;    }
 
       QString fretString(int fret, int string, bool ghost) const;   // returns a string with the text for fret
       QString durationString(TDuration::DurationType type, int dots) const;
 
-      // functions to cope with historic TAB's pecularities, like upside-down, bass string notations
+      // functions to cope with historic TAB's peculiarities, like upside-down, bass string notations
       int     physStringToVisual(int strg) const;                 // return the string in visual order from physical string
       int     visualStringToPhys(int line) const;                 // return the string in physical order from visual string
       qreal   physStringToYOffset(int strg) const;                // return the string Y offset (in sp, chord-relative)
@@ -315,27 +331,27 @@ class StaffType {
       int     numOfTabLedgerLines(int string) const;
 
       // properties getters (some getters require updated metrics)
-      qreal durationBoxH();
-      qreal durationBoxY();
+      qreal durationBoxH() const;
+      qreal durationBoxY() const;
 
-      const QFont&  durationFont()           { return _durationFont;     }
+      const QFont&  durationFont() const     { return _durationFont;     }
       const QString durationFontName() const { return _durationFonts[_durationFontIdx].displayName; }
       qreal durationFontSize() const      { return _durationFontSize;   }
       qreal durationFontUserY() const     { return _durationFontUserY;  }
-      qreal durationFontYOffset()         { setDurationMetrics(); return _durationYOffset + _durationFontUserY * SPATIUM20; }
-      qreal durationGridYOffset()         { setDurationMetrics(); return _durationGridYOffset;}
-      qreal fretBoxH()                    { setFretMetrics(); return _fretBoxH; }
-      qreal fretBoxY()                    { setFretMetrics(); return _fretBoxY + _fretFontUserY * SPATIUM20; }
+      qreal durationFontYOffset() const        { setDurationMetrics(); return _durationYOffset + _durationFontUserY * SPATIUM20; }
+      qreal durationGridYOffset() const        { setDurationMetrics(); return _durationGridYOffset;}
+      qreal fretBoxH() const              { setFretMetrics(); return _fretBoxH; }
+      qreal fretBoxY() const              { setFretMetrics(); return _fretBoxY + _fretFontUserY * SPATIUM20; }
 
       // 2 methods to return the size of a box masking lines under a fret mark
-      qreal fretMaskH()                   { return _lineDistance.val() * SPATIUM20; }
-      qreal fretMaskY()                   { return (_onLines ? -0.5 : -1.0) * _lineDistance.val() * SPATIUM20; }
+      qreal fretMaskH() const             { return _lineDistance.val() * SPATIUM20; }
+      qreal fretMaskY() const             { return (_onLines ? -0.5 : -1.0) * _lineDistance.val() * SPATIUM20; }
 
       const QFont&  fretFont() const      { return _fretFont;           }
       const QString fretFontName() const  { return _fretFonts[_fretFontIdx].displayName; }
       qreal fretFontSize() const          { return _fretFontSize;       }
       qreal fretFontUserY() const         { return _fretFontUserY;      }
-      qreal fretFontYOffset()             { setFretMetrics(); return _fretYOffset + _fretFontUserY * SPATIUM20; }
+      qreal fretFontYOffset() const       { setFretMetrics(); return _fretYOffset + _fretFontUserY * SPATIUM20; }
       bool  genDurations() const          { return _genDurations;       }
       bool  linesThrough() const          { return _linesThrough;       }
       TablatureMinimStyle minimStyle() const    { return _minimStyle;   }
@@ -345,7 +361,7 @@ class StaffType {
       bool  stemsDown() const             { return _stemsDown;          }
       bool  stemThrough() const           { return _stemsThrough;       }
       bool  upsideDown() const            { return _upsideDown;         }
-      bool  showTabFingering() const      { return _showTabFingering;   }
+      bool  showTabFingering() const            { return _showTabFingering;         }
       bool  useNumbers() const            { return _useNumbers;         }
       bool  showBackTied() const          { return _showBackTied;       }
 
@@ -365,7 +381,7 @@ class StaffType {
       void  setStemsDown(bool val)        { _stemsDown = val;           }
       void  setStemsThrough(bool val)     { _stemsThrough = val;        }
       void  setUpsideDown(bool val)       { _upsideDown = val;          }
-      void  setShowTabFingering (bool val) { _showTabFingering = val;   }
+      void  setShowTabFingering (bool val) { _showTabFingering = val;         }
       void  setUseNumbers(bool val)       { _useNumbers = val; _fretMetricsValid = false; }
       void  setShowBackTied(bool val)     { _showBackTied = val;        }
 
@@ -376,12 +392,17 @@ class StaffType {
       QPointF chordStemPosBeam(const  Chord*) const;
       qreal   chordStemLength(const Chord*) const;
 
+      bool isTabStaff() const  { return _group == StaffGroup::TAB; }
+      bool isDrumStaff() const { return _group == StaffGroup::PERCUSSION; }
       // static functions for font config files
       static QList<QString> fontNames(bool bDuration);
       static bool fontData(bool bDuration, int nIdx, QString *pFamily, QString *pDisplayName, qreal * pSize, qreal *pYOff);
 
       static void initStaffTypes();
       static const std::vector<StaffType>& presets() { return _presets; }
+      static QString scheme2userName(NoteHeadScheme ns);
+      static QString scheme2name(NoteHeadScheme ns);
+      static NoteHeadScheme name2scheme(QString name);
       };
 
 //---------------------------------------------------------
@@ -396,26 +417,26 @@ enum class TabBeamGrid : char {
       NUM_OF
       };
 
-class TabDurationSymbol : public Element {
+class TabDurationSymbol final : public Element {
       qreal       _beamLength;      // if _grid==MEDIALFINAL, length of the beam toward previous grid element
       int         _beamLevel;       // if _grid==MEDIALFINAL, the number of beams
       TabBeamGrid _beamGrid;        // value for special 'English' grid display
-      StaffType*  _tab;
+      const StaffType*  _tab;
       QString     _text;
 
    public:
       TabDurationSymbol(Score* s);
-      TabDurationSymbol(Score* s, StaffType* tab, TDuration::DurationType type, int dots);
+      TabDurationSymbol(Score* s, const StaffType* tab, TDuration::DurationType type, int dots);
       TabDurationSymbol(const TabDurationSymbol&);
       virtual TabDurationSymbol* clone() const  { return new TabDurationSymbol(*this); }
       virtual void draw(QPainter*) const;
       virtual bool isEditable() const           { return false; }
       virtual void layout();
-      virtual Element::Type type() const        { return Element::Type::TAB_DURATION_SYMBOL; }
+      virtual ElementType type() const        { return ElementType::TAB_DURATION_SYMBOL; }
 
       TabBeamGrid beamGrid()                    { return _beamGrid; }
       void layout2();               // second step of layout: after horiz. pos. are defined, compute width of 'grid beams'
-      void setDuration(TDuration::DurationType type, int dots, StaffType* tab) {
+      void setDuration(TDuration::DurationType type, int dots, const StaffType* tab) {
             _tab = tab;
             _text = tab->durationString(type, dots);
             }
