@@ -64,7 +64,6 @@ static Interval keydiff2Interval(Key oKey, Key nKey, TransposeDirection dir)
             if (chromatic == -12)
                   chromatic = 0;
             }
-qDebug("TransposeByKey %d -> %d   chromatic %d diatonic %d", int(oKey), int(nKey), chromatic, diatonic);
       return Interval(diatonic, chromatic);
       }
 
@@ -241,9 +240,9 @@ bool Score::transpose(TransposeMode mode, TransposeDirection direction, Key trKe
   int transposeInterval, bool trKeys, bool transposeChordNames, bool useDoubleSharpsFlats)
       {
       bool rangeSelection = selection().isRange();
-      int startStaffIdx = 0;
-      int endStaffIdx   = 0;
-      int startTick     = 0;
+      int startStaffIdx   = 0;
+      int endStaffIdx     = 0;
+      Fraction startTick  = Fraction(0,1);
       if (rangeSelection) {
             startStaffIdx = selection().staffStart();
             endStaffIdx   = selection().staffEnd();
@@ -260,9 +259,9 @@ bool Score::transpose(TransposeMode mode, TransposeDirection direction, Key trKe
                   Key key = Key::C;
                   for (int i = startStaffIdx; i < endStaffIdx; ++i) {
                         Staff* s = staff(i);
-                        if (s->isPitchedStaff()) {
+                        if (s->isPitchedStaff(startTick)) {
                               key = s->key(startTick);
-                              if (!styleB(StyleIdx::concertPitch)) {
+                              if (!styleB(Sid::concertPitch)) {
                                     int diff = s->part()->instrument(startTick)->transpose().chromatic;
                                     if (diff)
                                           key = transposeKey(key, diff);
@@ -306,10 +305,10 @@ bool Score::transpose(TransposeMode mode, TransposeDirection direction, Key trKe
 
       if (_selection.isList()) {
             foreach (Element* e, _selection.uniqueElements()) {
-                  if (!e->staff() || e->staff()->staffType()->group() == StaffGroup::PERCUSSION)
+                  if (!e->staff() || e->staff()->staffType(e->tick())->group() == StaffGroup::PERCUSSION)
                         continue;
-                  if (e->type() == Element::Type::NOTE) {
-                        Note* note = static_cast<Note*>(e);
+                  if (e->isNote()) {
+                        Note* note = toNote(e);
                         if (mode == TransposeMode::DIATONICALLY)
                               note->transposeDiatonic(transposeInterval, trKeys, useDoubleSharpsFlats);
                         else {
@@ -317,17 +316,15 @@ bool Score::transpose(TransposeMode mode, TransposeDirection direction, Key trKe
                                     return false;
                               }
                         }
-                  else if ((e->type() == Element::Type::HARMONY) && transposeChordNames) {
-                        Harmony* h  = static_cast<Harmony*>(e);
+                  else if (e->isHarmony() && transposeChordNames) {
+                        Harmony* h  = toHarmony(e);
                         int rootTpc, baseTpc;
                         if (mode == TransposeMode::DIATONICALLY) {
-                              int tick = 0;
-                              if (h->parent()->type() == Element::Type::SEGMENT)
-                                    tick = static_cast<Segment*>(h->parent())->tick();
-                              else if (h->parent()->type() == Element::Type::FRET_DIAGRAM
-                                 && h->parent()->parent()->type() == Element::Type::SEGMENT) {
-                                    tick = static_cast<Segment*>(h->parent()->parent())->tick();
-                                    }
+                              Fraction tick = Fraction(0,1);
+                              if (h->parent()->isSegment())
+                                    tick = toSegment(h->parent())->tick();
+                              else if (h->parent()->isFretDiagram() && h->parent()->parent()->isSegment())
+                                    tick = toSegment(h->parent()->parent())->tick();
                               Key key = !h->staff() ? Key::C : h->staff()->key(tick);
                               rootTpc = transposeTpcDiatonicByKey(h->rootTpc(),
                                           transposeInterval, key, trKeys, useDoubleSharpsFlats);
@@ -340,8 +337,8 @@ bool Score::transpose(TransposeMode mode, TransposeDirection direction, Key trKe
                               }
                         undoTransposeHarmony(h, rootTpc, baseTpc);
                         }
-                  else if ((e->type() == Element::Type::KEYSIG) && mode != TransposeMode::DIATONICALLY && trKeys) {
-                        KeySig* ks     = static_cast<KeySig*>(e);
+                  else if (e->isKeySig() && mode != TransposeMode::DIATONICALLY && trKeys) {
+                        KeySig* ks     = toKeySig(e);
                         if (!ks->isCustom() && !ks->isAtonal()) {
                               Key key        = st->key(ks->tick());
                               KeySigEvent ke = ks->keySigEvent();
@@ -360,13 +357,13 @@ bool Score::transpose(TransposeMode mode, TransposeDirection direction, Key trKe
       QList<Staff*> sl;
       for (int staffIdx = _selection.staffStart(); staffIdx < _selection.staffEnd(); ++staffIdx) {
             Staff* s = staff(staffIdx);
-            if (s->staffType()->group() == StaffGroup::PERCUSSION)      // ignore percussion staff
+            if (s->staffType(Fraction(0,1))->group() == StaffGroup::PERCUSSION)      // ignore percussion staff
                   continue;
             if (sl.contains(s))
                   continue;
             bool alreadyThere = false;
             for (Staff* s2 : sl) {
-                  if (s2 == s || (s2->linkedStaves() && s2->linkedStaves()->staves().contains(s))) {
+                  if (s2 == s || (s2->links() && s2->links()->contains(s))) {
                         alreadyThere = true;
                         break;
                         }
@@ -388,20 +385,22 @@ bool Score::transpose(TransposeMode mode, TransposeDirection direction, Key trKe
       // if range starts with first CR of measure
       // then start looping from very beginning of measure
       // so we include key signature and can transpose that if requested
-      if (!s1->rtick())
+      if (s1->rtick().isZero())
             s1 = s1->measure()->first();
       Segment* s2 = _selection.endSegment();
       for (Segment* segment = s1; segment && segment != s2; segment = segment->next1()) {
-            for (int st : tracks) {
-                  if (staff(st/VOICES)->staffType()->group() == StaffGroup::PERCUSSION)
+            if (!segment->enabled())
+                  continue;
+            for (int track : tracks) {
+                  if (staff(track/VOICES)->staffType(s1->tick())->group() == StaffGroup::PERCUSSION)
                         continue;
-                  Element* e = segment->element(st);
+                  Element* e = segment->element(track);
                   if (!e)
                         continue;
 
-                  if (e->type() == Element::Type::CHORD) {
-                        Chord* chord = static_cast<Chord*>(e);
-                        QList<Note*> nl = chord->notes();
+                  if (e->isChord()) {
+                        Chord* chord = toChord(e);
+                        std::vector<Note*> nl = chord->notes();
                         for (Note* n : nl) {
                               if (mode == TransposeMode::DIATONICALLY)
                                     n->transposeDiatonic(transposeInterval, trKeys, useDoubleSharpsFlats);
@@ -421,67 +420,68 @@ bool Score::transpose(TransposeMode mode, TransposeDirection direction, Key trKe
                                     }
                               }
                         }
-                  else if (e->type() == Element::Type::KEYSIG && trKeys && mode != TransposeMode::DIATONICALLY) {
+                  else if (e->isKeySig() && trKeys && mode != TransposeMode::DIATONICALLY) {
+                        bool startKey = segment->tick() == s1->tick();
                         QList<ScoreElement*> ll = e->linkList();
-                        for (ScoreElement* e : ll) {
-                              KeySig* ks = static_cast<KeySig*>(e);
+                        for (ScoreElement* scoreElement : ll) {
+                              KeySig* ks = toKeySig(scoreElement);
+                              bool addKey = ks->isChange();
                               if (!ks->isCustom() && !ks->isAtonal()) {
                                     Key nKey = transposeKey(ks->key(), interval);
                                     KeySigEvent ke = ks->keySigEvent();
                                     ke.setKey(nKey);
-                                    undo(new ChangeKeySig(ks, ke, ks->showCourtesy()));
+                                    undo(new ChangeKeySig(ks, ke, ks->showCourtesy(), startKey || addKey));
                                     }
                               }
                         }
                   }
             if (transposeChordNames) {
                   foreach (Element* e, segment->annotations()) {
-                        if ((e->type() != Element::Type::HARMONY) || (!tracks.contains(e->track())))
+                        if ((e->type() != ElementType::HARMONY) || (!tracks.contains(e->track())))
                               continue;
-                        Harmony* h  = static_cast<Harmony*>(e);
+                        Harmony* hh  = toHarmony(e);
                         int rootTpc, baseTpc;
-                        if (mode == TransposeMode::DIATONICALLY) {
-                              int tick = segment->tick();
-                              Key key = !h->staff() ? Key::C : h->staff()->key(tick);
-                              rootTpc = transposeTpcDiatonicByKey(h->rootTpc(),
-                                          transposeInterval, key, trKeys, useDoubleSharpsFlats);
-                              baseTpc = transposeTpcDiatonicByKey(h->baseTpc(),
-                                          transposeInterval, key, trKeys, useDoubleSharpsFlats);
-                              }
-                        else {
-                              rootTpc = transposeTpc(h->rootTpc(), interval, useDoubleSharpsFlats);
-                              baseTpc = transposeTpc(h->baseTpc(), interval, useDoubleSharpsFlats);
-                              }
                         // undoTransposeHarmony does not do links
                         // because it is also used to handle transposing instruments
                         // and score / parts could be in different concert pitch states
-                        for (ScoreElement* e : h->linkList())
-                              undoTransposeHarmony(static_cast<Harmony*>(e), rootTpc, baseTpc);
+                        for (ScoreElement* se : hh->linkList()) {
+                              Harmony* h = toHarmony(se);
+                              if (mode == TransposeMode::DIATONICALLY) {
+                                    Fraction tick = segment->tick();
+                                    Key key = !h->staff() ? Key::C : h->staff()->key(tick);
+                                    rootTpc = transposeTpcDiatonicByKey(h->rootTpc(),
+                                                transposeInterval, key, trKeys, useDoubleSharpsFlats);
+                                    baseTpc = transposeTpcDiatonicByKey(h->baseTpc(),
+                                                transposeInterval, key, trKeys, useDoubleSharpsFlats);
+                                    }
+                              else {
+                                    rootTpc = transposeTpc(h->rootTpc(), interval, useDoubleSharpsFlats);
+                                    baseTpc = transposeTpc(h->baseTpc(), interval, useDoubleSharpsFlats);
+                                    }
+                              undoTransposeHarmony(h, rootTpc, baseTpc);
+                              }
                         }
                   }
             }
       //
       // create missing key signatures
       //
-      if (trKeys && (mode != TransposeMode::DIATONICALLY) && (s1->tick() == 0)) {
-//            Segment* seg = firstMeasure()->findSegment(Segment::Type::KeySig, 0);
+      if (trKeys && (mode != TransposeMode::DIATONICALLY) && (s1->tick() == Fraction(0,1))) {
             Key nKey = transposeKey(Key::C, interval);
-//            if (seg == 0) {
-                  for (int st : tracks) {
-                        if (st % VOICES)
-                              continue;
-                        Segment* seg = firstMeasure()->undoGetSegment(Segment::Type::KeySig, 0);
-                        KeySig* ks = static_cast<KeySig*>(seg->element(st));
-                        if (!ks) {
-                              ks = new KeySig(this);
-                              ks->setTrack(st);
-                              ks->setKey(nKey);
-                              ks->setParent(seg);
-                              undoAddElement(ks);
-                              }
+            for (int track : tracks) {
+                  if (track % VOICES)
+                        continue;
+                  Segment* seg = firstMeasure()->undoGetSegmentR(SegmentType::KeySig, Fraction(0,1));
+                  KeySig* ks = toKeySig(seg->element(track));
+                  if (!ks) {
+                        ks = new KeySig(this);
+                        ks->setTrack(track);
+                        ks->setKey(nKey);
+                        ks->setParent(seg);
+                        undoAddElement(ks);
                         }
                   }
-//            }
+            }
       return true;
       }
 
@@ -490,40 +490,41 @@ bool Score::transpose(TransposeMode mode, TransposeDirection direction, Key trKe
 //    key -   -7(Cb) - +7(C#)
 //---------------------------------------------------------
 
-void Score::transposeKeys(int staffStart, int staffEnd, int tickStart, int tickEnd, const Interval& interval, bool useInstrument, bool flip)
+void Score::transposeKeys(int staffStart, int staffEnd, const Fraction& ts, const Fraction& tickEnd, const Interval& interval, bool useInstrument, bool flip)
       {
+      Fraction tickStart(ts);
       Interval firstInterval = interval;
       Interval segmentInterval = interval;
+      if (tickStart < Fraction(0,1))            // -1 and 0 are valid values to indicate start of score
+            tickStart = Fraction(0,1);
       for (int staffIdx = staffStart; staffIdx < staffEnd; ++staffIdx) {
             Staff* st = staff(staffIdx);
-            if (st->staffType()->group() == StaffGroup::PERCUSSION)
+            if (st->staffType(tickStart)->group() == StaffGroup::PERCUSSION)
                   continue;
 
-            bool createKey = tickStart <= 0;    // 0 and -1 are both valid values to indicate start of score
-            for (Segment* s = firstSegment(Segment::Type::KeySig); s; s = s->next1(Segment::Type::KeySig)) {
-                  if (s->tick() < tickStart)
+            bool createKey = tickStart.isZero();
+            for (Segment* s = firstSegment(SegmentType::KeySig); s; s = s->next1(SegmentType::KeySig)) {
+                  if (!s->enabled() || s->tick() < tickStart)
                         continue;
-                  if (tickEnd != -1 && s->tick() >= tickEnd)
+                  if (tickEnd != Fraction(-1,1) && s->tick() >= tickEnd)
                         break;
                   if (useInstrument) {
                         segmentInterval = st->part()->instrument(s->tick())->transpose();
                         if (flip)
                               segmentInterval.flip();
                         }
-                  KeySig* ks = static_cast<KeySig*>(s->element(staffIdx * VOICES));
-                  if (!ks)
+                  KeySig* ks = toKeySig(s->element(staffIdx * VOICES));
+                  if (!ks || ks->generated())
                         continue;
-                  if (ks->generated())
-                        continue;
-                  if (s->tick() == 0)
+                  if (s->tick().isZero())
                         createKey = false;
                   if (!ks->isCustom() && !ks->isAtonal()) {
                         Key key  = st->key(s->tick());
                         Key nKey = transposeKey(key, segmentInterval);
                         // remove initial C major key signatures
-                        if (nKey == Key::C && s->tick() == 0) {
+                        if (nKey == Key::C && s->tick().isZero()) {
                               undo(new RemoveElement(ks));
-                              if (s->isEmpty())
+                              if (s->empty())
                                     undo(new RemoveElement(s));
                               }
                         else {
@@ -540,8 +541,9 @@ void Score::transposeKeys(int staffStart, int staffEnd, int tickStart, int tickE
                   ke.setKey(nKey);
                   KeySig* ks = new KeySig(this);
                   ks->setTrack(staffIdx * VOICES);
-                  ks->setKey(nKey);
-                  Segment* seg = firstMeasure()->undoGetSegment(Segment::Type::KeySig, 0);
+                  ks->setKeySigEvent(ke);
+                  Segment* seg = firstMeasure()->undoGetSegmentR(SegmentType::KeySig, Fraction(0,1));
+                  seg->setHeader(true);
                   ks->setParent(seg);
                   undoAddElement(ks);
                   }
@@ -563,9 +565,9 @@ void Score::transposeSemitone(int step)
 
       TransposeDirection dir = step > 0 ? TransposeDirection::UP : TransposeDirection::DOWN;
 
-      int keyType = int(staff(0)->key(0)) + 7;   // ??
+      int keyType = int(staff(0)->key(Fraction(0,1))) + 7;   // ??
 
-      int intervalList[15][2] = {
+      int intervalListArray[15][2] = {
             // up - down
             { 1, 1 },  // Cb
             { 1, 1 },  // Gb
@@ -584,19 +586,15 @@ void Score::transposeSemitone(int step)
             { 1, 1 }   // C#
             };
 
-      int interval = intervalList[keyType][step > 0 ? 0 : 1];
+      const int interval = intervalListArray[keyType][step > 0 ? 0 : 1];
 
       cmdSelectAll();
-      startCmd();
       if (!transpose(TransposeMode::BY_INTERVAL, dir, Key::C, interval, true, true, false)) {
-            endCmd(true);
             qDebug("Score::transposeSemitone: failed");
-            // TODO: popup message?
+            // TODO: set error message
             }
       else {
             deselectAll();
-            setLayoutAll(true);
-            endCmd(false);
             }
       }
 
@@ -608,9 +606,9 @@ void Note::transposeDiatonic(int interval, bool keepAlterations, bool useDoubleA
       {
       // compute note current absolute step
       int alter;
-      int tick     = chord()->segment()->tick();
-      Key key      = staff() ? staff()->key(tick) : Key::C;
-      int absStep  = pitch2absStepByKey(epitch(), tpc(), key, &alter);
+      Fraction tick = chord()->segment()->tick();
+      Key key       = staff() ? staff()->key(tick) : Key::C;
+      int absStep   = pitch2absStepByKey(epitch(), tpc(), key, &alter);
 
       // get pitch and tcp corresponding to unaltered degree for this key
       int newPitch = absStep2pitchByKey(absStep + interval, key);
@@ -671,8 +669,10 @@ void Note::transposeDiatonic(int interval, bool keepAlterations, bool useDoubleA
 //   transpositionChanged
 //---------------------------------------------------------
 
-void Score::transpositionChanged(Part* part, Interval oldV, int tickStart, int tickEnd)
+void Score::transpositionChanged(Part* part, Interval oldV, Fraction tickStart, Fraction tickEnd)
       {
+      if (tickStart == Fraction(-1,1))
+            tickStart = Fraction(0,1);
       Interval v = part->instrument(tickStart)->transpose();
       v.flip();
       Interval diffV(oldV.chromatic + v.chromatic);
@@ -687,45 +687,46 @@ void Score::transpositionChanged(Part* part, Interval oldV, int tickStart, int t
                   continue;
             scores.append(score);
             Part* lp = ls->part();
-            if (!score->styleB(StyleIdx::concertPitch))
+            if (!score->styleB(Sid::concertPitch))
                   score->transposeKeys(lp->startTrack() / VOICES, lp->endTrack() / VOICES, tickStart, tickEnd, diffV);
             }
 
       // now transpose notes and chord symbols
-      for (Segment* s = firstSegment(Segment::Type::ChordRest); s; s = s->next1(Segment::Type::ChordRest)) {
+      for (Segment* s = firstSegment(SegmentType::ChordRest); s; s = s->next1(SegmentType::ChordRest)) {
             if (s->tick() < tickStart)
                   continue;
-            if (tickEnd != -1 && s->tick() >= tickEnd)
+            if (tickEnd != Fraction(-1,1) && s->tick() >= tickEnd)
                   break;
             for (Staff* st : *part->staves()) {
-                  if (st->staffType()->group() == StaffGroup::PERCUSSION)
+                  if (st->staffType(tickStart)->group() == StaffGroup::PERCUSSION)
                         continue;
                   int t1 = st->idx() * VOICES;
                   int t2 = t1 + VOICES;
                   for (int track = t1; track < t2; ++track) {
-                        Chord* c = static_cast<Chord*>(s->element(track));
-                        if (c && c->type() == Element::Type::CHORD) {
+                        Element* e = s->element(track);
+                        if (e && e->isChord()) {
+                              Chord* c = toChord(e);
                               for (Chord* gc : c->graceNotes()) {
                                     for (Note* n : gc->notes()) {
                                           int tpc = transposeTpc(n->tpc1(), v, true);
-                                          n->undoSetTpc2(tpc);
+                                          n->undoChangeProperty(Pid::TPC2, tpc);
                                           }
                                     }
                               for (Note* n : c->notes()) {
                                     int tpc = transposeTpc(n->tpc1(), v, true);
-                                    n->undoSetTpc2(tpc);
+                                    n->undoChangeProperty(Pid::TPC2, tpc);
                                     }
                               }
                         // find chord symbols
-                        for (Element* e : s->annotations()) {
-                              if (e->track() != track || e->type() != Element::Type::HARMONY)
+                        for (Element* element : s->annotations()) {
+                              if (element->track() != track || element->type() != ElementType::HARMONY)
                                     continue;
-                              Harmony* h  = static_cast<Harmony*>(e);
+                              Harmony* h  = toHarmony(element);
                               int rootTpc = transposeTpc(h->rootTpc(), diffV, false);
                               int baseTpc = transposeTpc(h->baseTpc(), diffV, false);
-                              for (ScoreElement* e : h->linkList()) {
-                                    if (!e->score()->styleB(StyleIdx::concertPitch))
-                                          undoTransposeHarmony(static_cast<Harmony*>(e), rootTpc, baseTpc);
+                              for (ScoreElement* scoreElement : h->linkList()) {
+                                    if (!scoreElement->score()->styleB(Sid::concertPitch))
+                                          undoTransposeHarmony(toHarmony(scoreElement), rootTpc, baseTpc);
                                     }
                               }
                         }
