@@ -1,9 +1,8 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
 //
-//  Copyright (C) 2010 Werner Schweer and others
+//  Copyright (C) 2010-2016 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -46,7 +45,6 @@ DrumTools::DrumTools(QWidget* parent)
       drumset = 0;
       _score  = 0;
       setObjectName("drum-tools");
-      setWindowTitle(tr("Drum Tools"));
       setAllowedAreas(Qt::DockWidgetAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea));
 
       QWidget* w = new QWidget(this);
@@ -60,20 +58,18 @@ DrumTools::DrumTools(QWidget* parent)
       pitchName = new QLabel;
       pitchName->setAlignment(Qt::AlignCenter);
       pitchName->setWordWrap(true);
-      pitchName->setContentsMargins(25, 10, 25, 10);
+      pitchName->setContentsMargins(25, 0, 25, 0);
       layout1->addWidget(pitchName);
       QHBoxLayout* buttonLayout = new QHBoxLayout;
       buttonLayout->setContentsMargins(25, 10, 25, 10);
-      QToolButton* tb = new QToolButton;
-      tb->setText(tr("Edit Drumset"));
-      tb->setMinimumWidth(100);
-      tb->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
-      buttonLayout->addWidget(tb);
+      editButton = new QToolButton;
+      editButton->setMinimumWidth(100);
+      editButton->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
+      buttonLayout->addWidget(editButton);
       layout1->addLayout(buttonLayout);
       layout->addLayout(layout1);
 
       drumPalette = new Palette;
-      drumPalette->setName(tr("Drums"));
       drumPalette->setMag(0.8);
       drumPalette->setSelectable(true);
       drumPalette->setGrid(28, 60);
@@ -86,10 +82,10 @@ DrumTools::DrumTools(QWidget* parent)
       w = new QWidget(this);
       setTitleBarWidget(w);
       titleBarWidget()->hide();
-      connect(tb, SIGNAL(clicked()), SLOT(editDrumset()));
+      connect(editButton, SIGNAL(clicked()), SLOT(editDrumset()));
       void boxClicked(int);
       connect(drumPalette, SIGNAL(boxClicked(int)), SLOT(drumNoteSelected(int)));
-
+      retranslate();
       drumPalette->setContextMenuPolicy(Qt::PreventContextMenu);
       }
 
@@ -97,6 +93,17 @@ DrumTools::DrumTools(QWidget* parent)
 //   updateDrumset
 //---------------------------------------------------------
 
+void DrumTools::retranslate()
+      {
+      setWindowTitle(tr("Drumset Tools"));
+      editButton->setText(tr("Edit Drumset"));
+      drumPalette->setName(tr("Drumset"));
+      }
+
+//---------------------------------------------------------
+//   updateDrumset
+//---------------------------------------------------------
+
 void DrumTools::updateDrumset(const Drumset* ds)
       {
       drumPalette->clear();
@@ -104,19 +111,17 @@ void DrumTools::updateDrumset(const Drumset* ds)
       if (!drumset)
             return;
       double _spatium = gscore->spatium();
-      const auto d = ds->drumsByIndex();
-      for (int n = 0; n < DRUM_INSTRUMENTS; ++n) {
-            DrumInstrument di = d[n];
-            if (!drumset->isValid(di.pitch))
+      for (int pitch = 0; pitch < 128; ++pitch) {
+            if (!drumset->isValid(pitch))
                   continue;
             bool up;
-            int line      = di.line;
-            NoteHead::Group noteHead  = di.notehead;
-            int voice     = di.voice;
-            MScore::Direction dir = di.stemDirection;
-            if (dir == MScore::Direction::UP)
+            int line      = drumset->line(pitch);
+            NoteHead::Group noteHead  = drumset->noteHead(pitch);
+            int voice     = drumset->voice(pitch);
+            Direction dir = drumset->stemDirection(pitch);
+            if (dir == Direction::UP)
                   up = true;
-            else if (dir == MScore::Direction::DOWN)
+            else if (dir == Direction::DOWN)
                   up = false;
             else
                   up = line > 4;
@@ -126,31 +131,31 @@ void DrumTools::updateDrumset(const Drumset* ds)
             chord->setStemDirection(dir);
             chord->setUp(up);
             chord->setTrack(voice);
+            Stem* stem = new Stem(gscore);
+            stem->setLen((up ? -3.0 : 3.0) * _spatium);
+            chord->add(stem);
             Note* note = new Note(gscore);
+            note->setMark(true);
             note->setParent(chord);
             note->setTrack(voice);
-            note->setPitch(di.pitch);
+            note->setPitch(pitch);
             note->setTpcFromPitch();
             note->setLine(line);
             note->setPos(0.0, _spatium * .5 * line);
             note->setHeadGroup(noteHead);
             SymId noteheadSym = SymId::noteheadBlack;
             if (noteHead == NoteHead::Group::HEAD_CUSTOM)
-                  noteheadSym = drumset->noteHeads(di.pitch, NoteHead::Type::HEAD_QUARTER);
+                  noteheadSym = drumset->noteHeads(pitch, NoteHead::Type::HEAD_QUARTER);
             else
-                  noteheadSym = note->noteHead(true, di.notehead, NoteHead::Type::HEAD_QUARTER);
-
+                  noteheadSym = note->noteHead(true, noteHead, NoteHead::Type::HEAD_QUARTER);
+            
             note->setCachedNoteheadSym(noteheadSym); // we use the cached notehead so we don't recompute it at each layout
             chord->add(note);
-            Stem* stem = new Stem(gscore);
-            stem->setLen((up ? -3.0 : 3.0) * _spatium);
-            chord->add(stem);
-            stem->setPos(chord->stemPos());
-            int sc = di.shortcut;
+            int sc = drumset->shortcut(pitch);
             QString shortcut;
             if (sc)
                   shortcut = QChar(sc);
-            drumPalette->append(chord, qApp->translate("drumset", drumset->name(di.pitch).toUtf8().data()), shortcut);
+            drumPalette->append(chord, qApp->translate("drumset", drumset->name(pitch).toUtf8().data()), shortcut);
             }
       }
 
@@ -179,8 +184,8 @@ void DrumTools::editDrumset()
             _score->startCmd();
             _score->undo(new ChangeDrumset(staff->part()->instrument(), eds.drumset()));
             mscore->updateDrumTools(eds.drumset());
-            if (_score->undo()->active()) {
-                  _score->setLayoutAll(true);
+            if (_score->undoStack()->active()) {
+                  _score->setLayoutAll();
                   _score->endCmd();
                   }
             }
@@ -193,12 +198,12 @@ void DrumTools::editDrumset()
 void DrumTools::drumNoteSelected(int val)
       {
       Element* element = drumPalette->element(val);
-      if (element && element->type() == Element::Type::CHORD) {
+      if (element && element->type() == ElementType::CHORD) {
             Chord* ch        = static_cast<Chord*>(element);
             Note* note       = ch->downNote();
             int ticks        = MScore::defaultPlayDuration;
             int pitch        = note->pitch();
-            seq->startNote(staff->part()->instrument()->channel(0)->channel, pitch, 80, ticks, 0.0);
+            seq->startNote(staff->part()->instrument()->channel(0)->channel(), pitch, 80, ticks, 0.0);
 
             int track = (_score->inputState().track() / VOICES) * VOICES + element->track();
             _score->inputState().setTrack(track);
@@ -209,7 +214,8 @@ void DrumTools::drumNoteSelected(int val)
             getAction("voice-3")->setChecked(element->voice() == 2);
             getAction("voice-4")->setChecked(element->voice() == 3);
 
-            pitchName->setText(drumPalette->getCellName(val));
+            auto pitchCell = drumPalette->cellAt(val);
+            pitchName->setText(pitchCell->name);
             }
       }
 
@@ -219,10 +225,11 @@ int DrumTools::selectedDrumNote()
       if (idx < 0)
             return -1;
       Element* element = drumPalette->element(idx);
-      if (element && element->type() == Element::Type::CHORD) {
+      if (element && element->type() == ElementType::CHORD) {
             Chord* ch  = static_cast<Chord*>(element);
             Note* note = ch->downNote();
-            pitchName->setText(drumPalette->getCellName(idx));
+            auto pitchCell = drumPalette->cellAt(idx);
+            pitchName->setText(pitchCell->name);
             return note->pitch();
             }
       else {
@@ -230,5 +237,11 @@ int DrumTools::selectedDrumNote()
             }
       }
 
+void DrumTools::changeEvent(QEvent *event)
+      {
+      QDockWidget::changeEvent(event);
+      if (event->type() == QEvent::LanguageChange)
+            retranslate();
+      }
 }
 
