@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: editstaff.cpp 5149 2011-12-29 08:38:43Z wschweer $
 //
 //  Copyright (C) 2002-2010 Werner Schweer and others
 //
@@ -44,7 +43,7 @@ namespace Ms {
 //   EditStaff
 //---------------------------------------------------------
 
-EditStaff::EditStaff(Staff* s, int /*tick*/, QWidget* parent)
+EditStaff::EditStaff(Staff* s, const Fraction& /*tick*/, QWidget* parent)
    : QDialog(parent)
       {
       setObjectName("EditStaff");
@@ -52,12 +51,6 @@ EditStaff::EditStaff(Staff* s, int /*tick*/, QWidget* parent)
       setWindowFlags(this->windowFlags() & ~Qt::WindowContextHelpButtonHint);
       setModal(true);
 
-      const QIcon &editIcon = *icons[int(Icons::edit_ICON)];
-      minPitchASelect->setIcon(editIcon);
-      maxPitchASelect->setIcon(editIcon);
-      minPitchPSelect->setIcon(editIcon);
-      maxPitchPSelect->setIcon(editIcon);
-
       staff = nullptr;
       setStaff(s);
 
@@ -80,7 +73,10 @@ EditStaff::EditStaff(Staff* s, int /*tick*/, QWidget* parent)
       connect(nextButton,           SIGNAL(clicked()),            SLOT(gotoNextStaff()));
       connect(previousButton,       SIGNAL(clicked()),            SLOT(gotoPreviousStaff()));
 
-      addAction(getAction("local-help"));  // why is this needed?
+      nextButton->setIcon(*icons[int(Icons::arrowDown_ICON)]);
+      previousButton->setIcon(*icons[int(Icons::arrowUp_ICON)]);
+      
+      addAction(getAction("help"));  // why is this needed?
       }
 
 //---------------------------------------------------------
@@ -97,25 +93,26 @@ void EditStaff::setStaff(Staff* s)
       instrument        = *part->instrument(/*tick*/);
       Score* score      = part->score();
       staff             = new Staff(score);
-      staff->setSmall(orgStaff->small());
+      staff->setStaffType(Fraction(0,1), *orgStaff->staffType(Fraction(0,1)));
+      staff->setSmall(Fraction(0,1), orgStaff->small(Fraction(0,1)));
       staff->setInvisible(orgStaff->invisible());
       staff->setUserDist(orgStaff->userDist());
       staff->setColor(orgStaff->color());
-      staff->setStaffType(orgStaff->staffType());
       staff->setPart(part);
-      staff->setNeverHide(orgStaff->neverHide());
+      staff->setCutaway(orgStaff->cutaway());
+      staff->setHideWhenEmpty(orgStaff->hideWhenEmpty());
       staff->setShowIfEmpty(orgStaff->showIfEmpty());
-      staff->setUserMag(orgStaff->userMag());
+      staff->setUserMag(Fraction(0,1), orgStaff->userMag(Fraction(0,1)));
       staff->setHideSystemBarLine(orgStaff->hideSystemBarLine());
 
       // get tick range for instrument
       auto i = part->instruments()->upper_bound(0);   // tick
       if (i == part->instruments()->end())
-            _tickEnd = -1;
+            _tickEnd = Fraction(-1,1);
       else
-            _tickEnd = i->first;
+            _tickEnd = Fraction::fromTicks(i->first);
 #if 1
-      _tickStart = -1;
+      _tickStart = Fraction(-1,1);
 #else
       --i;
       if (i == part->instruments()->begin())
@@ -127,20 +124,21 @@ void EditStaff::setStaff(Staff* s)
       // set dlg controls
       spinExtraDistance->setValue(s->userDist() / score->spatium());
       invisible->setChecked(staff->invisible());
-      small->setChecked(staff->small());
+      small->setChecked(staff->small(Fraction(0,1)));
       color->setColor(s->color());
       partName->setText(part->partName());
-      neverHide->setChecked(staff->neverHide());
+      cutaway->setChecked(staff->cutaway());
+      hideMode->setCurrentIndex(int(staff->hideWhenEmpty()));
       showIfEmpty->setChecked(staff->showIfEmpty());
       hideSystemBarLine->setChecked(staff->hideSystemBarLine());
-      mag->setValue(staff->userMag() * 100.0);
+      mag->setValue(staff->userMag(Fraction(0,1)) * 100.0);
       updateStaffType();
       updateInstrument();
       updateNextPreviousButtons();
       }
 
 //---------------------------------------------------------
-//   closeEvent
+//   hideEvent
 //---------------------------------------------------------
 
 void EditStaff::hideEvent(QHideEvent* ev)
@@ -155,7 +153,7 @@ void EditStaff::hideEvent(QHideEvent* ev)
 
 void EditStaff::updateStaffType()
       {
-      StaffType* staffType = staff->staffType();
+      const StaffType* staffType = staff->staffType(Fraction(0,1));
       lines->setValue(staffType->lines());
       lineDistance->setValue(staffType->lineDistance().val());
       showClef->setChecked(staffType->genClef());
@@ -194,6 +192,7 @@ void EditStaff::updateInstrument()
       maxPitchA->setText(midiCodeToStr(_maxPitchA));
       minPitchP->setText(midiCodeToStr(_minPitchP));
       maxPitchP->setText(midiCodeToStr(_maxPitchP));
+      singleNoteDynamics->setChecked(instrument.singleNoteDynamics());
 
       // only show string data controls if instrument has strings
       int numStr = instrument.stringData() ? instrument.stringData()->strings() : 0;
@@ -241,7 +240,8 @@ void EditStaff::updateInterval(const Interval& iv)
 void EditStaff::updateNextPreviousButtons()
       {
       int staffIdx = orgStaff->idx();
-      nextButton->setEnabled(staffIdx < orgStaff->score()->nstaves() - 1);
+
+      nextButton->setEnabled(staffIdx < (orgStaff->score()->nstaves() - 1));
       previousButton->setEnabled(staffIdx != 0);
       }
 
@@ -338,15 +338,23 @@ void EditStaff::apply()
       instrument.setShortName(sn);
       instrument.setLongName(ln);
 
+      instrument.setSingleNoteDynamics(singleNoteDynamics->isChecked());
+
       bool inv       = invisible->isChecked();
-      ClefTypeList clefType = instrument.clefType(orgStaff->rstaff());
+      ClefTypeList clefType = orgStaff->defaultClefType();
       qreal userDist = spinExtraDistance->value();
-      bool nhide     = neverHide->isChecked();
       bool ifEmpty   = showIfEmpty->isChecked();
       bool hideSystemBL = hideSystemBarLine->isChecked();
+      bool cutAway      = cutaway->isChecked();
+      Staff::HideMode hideEmpty = Staff::HideMode(hideMode->currentIndex());
 
       QString newPartName = partName->text().simplified();
-      if (!(instrument == *part->instrument()) || part->partName() != newPartName) {
+
+      bool instrumentFieldChanged = !(instrument == *part->instrument());
+      if (instrumentFieldChanged)
+            clefType = instrument.clefType(orgStaff->rstaff());
+
+      if (instrumentFieldChanged || part->partName() != newPartName) {
             // instrument has changed
             Interval v1 = instrument.transpose();
             Interval v2 = part->instrument()->transpose();
@@ -357,27 +365,29 @@ void EditStaff::apply()
             if (v1 != v2)
                   score->transpositionChanged(part, v2, _tickStart, _tickEnd);
             }
-      orgStaff->undoChangeProperty(P_ID::MAG, mag->value() / 100.0);
-      orgStaff->undoChangeProperty(P_ID::COLOR, color->color());
-      orgStaff->undoChangeProperty(P_ID::SMALL, small->isChecked());
+      orgStaff->undoChangeProperty(Pid::MAG, mag->value() / 100.0);
+      orgStaff->undoChangeProperty(Pid::COLOR, color->color());
+      orgStaff->undoChangeProperty(Pid::SMALL, small->isChecked());
 
       if (inv != orgStaff->invisible()
          || clefType != orgStaff->defaultClefType()
          || userDist != orgStaff->userDist()
-         || nhide != orgStaff->neverHide()
+         || cutAway != orgStaff->cutaway()
+         || hideEmpty != orgStaff->hideWhenEmpty()
          || ifEmpty != orgStaff->showIfEmpty()
          || hideSystemBL != orgStaff->hideSystemBarLine()
          ) {
-            score->undo(new ChangeStaff(orgStaff, inv, clefType, userDist * score->spatium(), nhide, ifEmpty, hideSystemBL));
+            score->undo(new ChangeStaff(orgStaff, inv, clefType, userDist * score->spatium(), hideEmpty, ifEmpty, cutAway, hideSystemBL));
             }
 
-      if ( !(*orgStaff->staffType() == *staff->staffType()) ) {
+      if ( !(*orgStaff->staffType(Fraction(0,1)) == *staff->staffType(Fraction(0,1))) ) {
             // updateNeeded |= (orgStaff->staffGroup() == StaffGroup::TAB || staff->staffGroup() == StaffGroup::TAB);
-            score->undo()->push(new ChangeStaffType(orgStaff, *staff->staffType()));
+            score->undo(new ChangeStaffType(orgStaff, *staff->staffType(Fraction(0,1))));
             }
 
       score->update();
-      score->updateChannel();
+      score->masterScore()->updateChannel();
+      staff->score()->masterScore()->rebuildAndUpdateExpressive(MuseScore::synthesizer("Fluid"));
       }
 
 //---------------------------------------------------------
@@ -434,27 +444,31 @@ void EditStaff::maxPitchPClicked()
 
 void EditStaff::lineDistanceChanged()
       {
-      staff->staffType()->setLineDistance(Spatium(lineDistance->value()));
+      staff->staffType(Fraction(0,1))->setLineDistance(Spatium(lineDistance->value()));
       }
 
+//---------------------------------------------------------
+//   numOfLinesChanged
+//---------------------------------------------------------
+
 void EditStaff::numOfLinesChanged()
       {
-      staff->staffType()->setLines(lines->value());
+      staff->staffType(Fraction(0,1))->setLines(lines->value());
       }
 
 void EditStaff::showClefChanged()
       {
-      staff->staffType()->setGenClef(showClef->checkState() == Qt::Checked);
+      staff->staffType(Fraction(0,1))->setGenClef(showClef->checkState() == Qt::Checked);
       }
 
 void EditStaff::showTimeSigChanged()
       {
-      staff->staffType()->setGenTimesig(showTimesig->checkState() == Qt::Checked);
+      staff->staffType(Fraction(0,1))->setGenTimesig(showTimesig->checkState() == Qt::Checked);
       }
 
 void EditStaff::showBarlinesChanged()
       {
-      staff->staffType()->setShowBarlines(showBarlines->checkState() == Qt::Checked);
+      staff->staffType(Fraction(0,1))->setShowBarlines(showBarlines->checkState() == Qt::Checked);
       }
 
 //---------------------------------------------------------
@@ -530,16 +544,16 @@ void EditStaff::editStringDataClicked()
 
 static const char* g_cNoteName[] = {
       QT_TRANSLATE_NOOP("editstaff", "C"),
-      QT_TRANSLATE_NOOP("editstaff", "C#"),
+      QT_TRANSLATE_NOOP("editstaff", "C♯"),
       QT_TRANSLATE_NOOP("editstaff", "D"),
-      QT_TRANSLATE_NOOP("editstaff", "Eb"),
+      QT_TRANSLATE_NOOP("editstaff", "E♭"),
       QT_TRANSLATE_NOOP("editstaff", "E"),
       QT_TRANSLATE_NOOP("editstaff", "F"),
-      QT_TRANSLATE_NOOP("editstaff", "F#"),
+      QT_TRANSLATE_NOOP("editstaff", "F♯"),
       QT_TRANSLATE_NOOP("editstaff", "G"),
-      QT_TRANSLATE_NOOP("editstaff", "Ab"),
+      QT_TRANSLATE_NOOP("editstaff", "A♭"),
       QT_TRANSLATE_NOOP("editstaff", "A"),
-      QT_TRANSLATE_NOOP("editstaff", "Bb"),
+      QT_TRANSLATE_NOOP("editstaff", "B♭"),
       QT_TRANSLATE_NOOP("editstaff", "B")
       };
 
@@ -557,7 +571,7 @@ void EditStaff::showStaffTypeDialog()
       EditStaffType editor(this, staff);
       editor.setWindowModality(Qt::WindowModal);
       if (editor.exec()) {
-            staff->setStaffType(editor.getStaffType());
+            staff->setStaffType(Fraction(0,1), *editor.getStaffType());
             updateStaffType();
             }
       }
