@@ -12,6 +12,7 @@
 
 #include "editstafftype.h"
 #include "libmscore/part.h"
+#include "libmscore/mscore.h"
 #include "libmscore/score.h"
 #include "libmscore/staff.h"
 #include "libmscore/stringdata.h"
@@ -21,7 +22,7 @@
 
 namespace Ms {
 
-extern Score::FileError readScore(Score* score, QString name, bool ignoreVersionError);
+extern Score::FileError readScore(MasterScore* score, QString name, bool ignoreVersionError);
 
 const char* g_groupNames[STAFF_GROUP_MAX] = {
       QT_TRANSLATE_NOOP("staff group header name", "STANDARD STAFF"),
@@ -29,6 +30,22 @@ const char* g_groupNames[STAFF_GROUP_MAX] = {
       QT_TRANSLATE_NOOP("staff group header name", "TABLATURE STAFF")
       };
 
+//---------------------------------------------------------
+//   noteHeadSchemes
+//---------------------------------------------------------
+
+NoteHeadScheme noteHeadSchemes[] = {
+      NoteHeadScheme::HEAD_NORMAL,
+      NoteHeadScheme::HEAD_PITCHNAME,
+      NoteHeadScheme::HEAD_PITCHNAME_GERMAN,
+      NoteHeadScheme::HEAD_SOLFEGE,
+      NoteHeadScheme::HEAD_SOLFEGE_FIXED,
+      NoteHeadScheme::HEAD_SHAPE_NOTE_4,
+      NoteHeadScheme::HEAD_SHAPE_NOTE_7_AIKIN,
+      NoteHeadScheme::HEAD_SHAPE_NOTE_7_FUNK,
+      NoteHeadScheme::HEAD_SHAPE_NOTE_7_WALKER
+      };
+
 //---------------------------------------------------------
 //   EditStaffType
 //---------------------------------------------------------
@@ -41,7 +58,7 @@ EditStaffType::EditStaffType(QWidget* parent, Staff* st)
       setupUi(this);
 
       staff     = st;
-      staffType = *staff->staffType();
+      staffType = *staff->staffType(Fraction(0,1));
       Instrument* instr = staff->part()->instrument();
 
       // template combo
@@ -63,18 +80,32 @@ EditStaffType::EditStaffType(QWidget* parent, Staff* st)
 
       // tab page configuration
       QList<QString> fontNames = StaffType::fontNames(false);
-      foreach (const QString& name, fontNames)   // fill fret font name combo
-            fretFontName->addItem(name);
+      foreach (const QString& fn, fontNames)   // fill fret font name combo
+            fretFontName->addItem(fn);
       fretFontName->setCurrentIndex(0);
       fontNames = StaffType::fontNames(true);
-      foreach(const QString& name, fontNames)   // fill duration font name combo
-            durFontName->addItem(name);
+      foreach(const QString& fn, fontNames)   // fill duration font name combo
+            durFontName->addItem(fn);
       durFontName->setCurrentIndex(0);
 
+      for (auto i : noteHeadSchemes)
+            noteHeadScheme->addItem(StaffType::scheme2userName(i), StaffType::scheme2name(i));
+
+      // load a sample standard score in preview
+      MasterScore* sc = new MasterScore(MScore::defaultStyle());
+      if (readScore(sc, QString(":/data/std_sample.mscx"), false) == Score::FileError::FILE_NO_ERROR)
+            standardPreview->setScore(sc);
+      else {
+            Q_ASSERT_X(false, "EditStaffType::EditStaffType", "Error in opening sample standard file for preview");
+            }
+
       // load a sample tabulature score in preview
-      Score* sc = new Score(MScore::defaultStyle());
+      sc = new MasterScore(MScore::defaultStyle());
       if (readScore(sc, QString(":/data/tab_sample.mscx"), false) == Score::FileError::FILE_NO_ERROR)
-            preview->setScore(sc);
+            tabPreview->setScore(sc);
+      else {
+            Q_ASSERT_X(false, "EditStaffType::EditStaffType", "Error in opening sample tab file for preview");
+            }
 
       setValues();
 
@@ -84,6 +115,7 @@ EditStaffType::EditStaffType(QWidget* parent, Staff* st)
       connect(showBarlines,   SIGNAL(toggled(bool)),              SLOT(updatePreview()));
       connect(genClef,        SIGNAL(toggled(bool)),              SLOT(updatePreview()));
       connect(genTimesig,     SIGNAL(toggled(bool)),              SLOT(updatePreview()));
+      connect(noteHeadScheme, SIGNAL(currentIndexChanged(int)),   SLOT(updatePreview()));
 
       connect(genKeysigPitched,           SIGNAL(toggled(bool)),  SLOT(updatePreview()));
       connect(showLedgerLinesPitched,     SIGNAL(toggled(bool)),  SLOT(updatePreview()));
@@ -114,8 +146,8 @@ EditStaffType::EditStaffType(QWidget* parent, Staff* st)
 
       connect(linesThroughRadio,    SIGNAL(toggled(bool)),              SLOT(updatePreview()));
       connect(onLinesRadio,         SIGNAL(toggled(bool)),              SLOT(updatePreview()));
-      connect(upsideDown,           SIGNAL(toggled(bool)),              SLOT(updatePreview()));
       connect(showTabFingering,     SIGNAL(toggled(bool)),              SLOT(updatePreview()));
+      connect(upsideDown,           SIGNAL(toggled(bool)),              SLOT(updatePreview()));
       connect(numbersRadio,         SIGNAL(toggled(bool)),              SLOT(updatePreview()));
       connect(showBackTied,         SIGNAL(toggled(bool)),              SLOT(updatePreview()));
 
@@ -158,10 +190,10 @@ void EditStaffType::setValues()
       blockSignals(true);
 
       StaffGroup group = staffType.group();
-      int idx = int(group);
-      stack->setCurrentIndex(idx);
-      groupName->setText(qApp->translate("staff group header name", g_groupNames[idx]));
-//      groupCombo->setCurrentIndex(idx);
+      int i = int(group);
+      stack->setCurrentIndex(i);
+      groupName->setText(qApp->translate("staff group header name", g_groupNames[i]));
+//      groupCombo->setCurrentIndex(i);
 
       name->setText(staffType.name());
       lines->setValue(staffType.lines());
@@ -175,11 +207,13 @@ void EditStaffType::setValues()
                   genKeysigPitched->setChecked(staffType.genKeysig());
                   showLedgerLinesPitched->setChecked(staffType.showLedgerLines());
                   stemlessPitched->setChecked(staffType.slashStyle());
+                  noteHeadScheme->setCurrentIndex(int(staffType.noteHeadScheme()));
                   break;
 
             case StaffGroup::TAB:
                   {
                   upsideDown->setChecked(staffType.upsideDown());
+                  showTabFingering->setChecked(staffType.showTabFingering());
                   int idx = fretFontName->findText(staffType.fretFontName(), Qt::MatchFixedString);
                   if (idx == -1)
                         idx = 0;          // if name not found, use first name
@@ -187,7 +221,6 @@ void EditStaffType::setValues()
                   fretFontSize->setValue(staffType.fretFontSize());
                   fretY->setValue(staffType.fretFontUserY());
 
-                  showTabFingering->setChecked(staffType.showTabFingering());
                   numbersRadio->setChecked(staffType.useNumbers());
                   lettersRadio->setChecked(!staffType.useNumbers());
                   onLinesRadio->setChecked(staffType.onLines());
@@ -349,6 +382,7 @@ void EditStaffType::setFromDlg()
             staffType.setGenKeysig(genKeysigPitched->isChecked());
             staffType.setShowLedgerLines(showLedgerLinesPitched->isChecked());
             staffType.setSlashStyle(stemlessPitched->isChecked());
+            staffType.setNoteHeadScheme(StaffType::name2scheme(noteHeadScheme->currentData().toString()));
             }
       if (staffType.group() == StaffGroup::PERCUSSION) {
             staffType.setGenKeysig(genKeysigPercussion->isChecked());
@@ -408,13 +442,13 @@ void EditStaffType::blockSignals(bool block)
       fretFontSize->blockSignals(block);
       fretY->blockSignals(block);
 
-      showTabFingering->blockSignals(block);
       numbersRadio->blockSignals(block);
       linesThroughRadio->blockSignals(block);
       onLinesRadio->blockSignals(block);
       showBackTied->blockSignals(block);
 
       upsideDown->blockSignals(block);
+      showTabFingering->blockSignals(block);
       valuesRepeatNever->blockSignals(block);
       valuesRepeatSystem->blockSignals(block);
       valuesRepeatMeasure->blockSignals(block);
@@ -503,8 +537,13 @@ void EditStaffType::tabStemThroughCompatibility(bool checked)
 void EditStaffType::updatePreview()
       {
       setFromDlg();
+      ExampleView* preview = nullptr;
+      if (staffType.group() == StaffGroup::TAB)
+             preview = tabPreview;
+      else if (staffType.group() == StaffGroup::STANDARD)
+             preview = standardPreview;
       if (preview) {
-            preview->score()->staff(0)->setStaffType(&staffType);
+            preview->score()->staff(0)->setStaffType(Fraction(0,1), staffType);
             preview->score()->doLayout();
             preview->updateAll();
             preview->update();
@@ -518,22 +557,22 @@ void EditStaffType::updatePreview()
 
 QString EditStaffType::createUniqueStaffTypeName(StaffGroup group)
       {
-      QString name;
+      QString sn;
       for (int idx = 1; ; ++idx) {
             switch (group) {
                   case StaffGroup::STANDARD:
-                        name = QString("Standard-%1 [*]").arg(idx);
+                        sn = QString("Standard-%1 [*]").arg(idx);
                         break;
                   case StaffGroup::PERCUSSION:
-                        name = QString("Perc-%1 [*]").arg(idx);
+                        sn = QString("Perc-%1 [*]").arg(idx);
                         break;
                   case StaffGroup::TAB:
-                        name = QString("Tab-%1 [*]").arg(idx);
+                        sn = QString("Tab-%1 [*]").arg(idx);
                         break;
                   }
             bool found = false;
             for (const StaffType& st : StaffType::presets()) {
-                  if (st.name() == name) {
+                  if (st.name() == sn) {
                         found = true;
                         break;
                         }
@@ -541,7 +580,7 @@ QString EditStaffType::createUniqueStaffTypeName(StaffGroup group)
             if (!found)
                   break;
             }
-      return name;
+      return sn;
       }
 
 //---------------------------------------------------------
