@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: instrdialog.cpp 5580 2012-04-27 15:36:57Z wschweer $
 //
 //  Copyright (C) 2002-2009 Werner Schweer and others
 //
@@ -35,6 +34,7 @@
 #include "libmscore/staff.h"
 #include "libmscore/stafftype.h"
 #include "libmscore/undo.h"
+#include "libmscore/bracketItem.h"
 
 namespace Ms {
 
@@ -56,6 +56,15 @@ InstrumentsDialog::InstrumentsDialog(QWidget* parent)
       readSettings();
       }
 
+//---------------------------------------------------------
+//   init
+//---------------------------------------------------------
+
+void InstrumentsDialog::init()
+      {
+      instrumentsWidget->init();
+      }
+
 //---------------------------------------------------------
 //   accept
 //---------------------------------------------------------
@@ -77,7 +86,7 @@ void InstrumentsDialog::on_saveButton_clicked()
          ".",
          tr("MuseScore Instruments") + " (*.xml)",
          0,
-         preferences.nativeDialogs ? QFileDialog::Options() : QFileDialog::DontUseNativeDialog
+         preferences.getBool(PREF_UI_APP_USENATIVEDIALOGS) ? QFileDialog::Options() : QFileDialog::DontUseNativeDialog
          );
       if (name.isEmpty())
             return;
@@ -88,24 +97,23 @@ void InstrumentsDialog::on_saveButton_clicked()
             info.setFile(info.filePath() + ext);
       QFile f(info.filePath());
       if (!f.open(QIODevice::WriteOnly)) {
-            QString s = tr("Open Instruments File\n%1\nfailed: ")
-               + QString(strerror(errno));
-            QMessageBox::critical(mscore, tr("Open Instruments File"), s.arg(f.fileName()));
+            QString s = tr("Open Instruments File\n%1\nfailed: %2").arg(f.fileName(), strerror(errno));
+            QMessageBox::critical(mscore, tr("Open Instruments File"), s);
             return;
             }
 
-      Xml xml(&f);
+      XmlWriter xml(0, &f);
       xml.header();
       xml.stag("museScore version=\"" MSC_VERSION "\"");
-      foreach(InstrumentGroup* g, instrumentGroups) {
+      for (InstrumentGroup* g : instrumentGroups) {
             xml.stag(QString("InstrumentGroup name=\"%1\" extended=\"%2\"").arg(g->name).arg(g->extended));
-            foreach(InstrumentTemplate* t, g->instrumentTemplates)
+            for (InstrumentTemplate* t : g->instrumentTemplates)
                   t->write(xml);
             xml.etag();
             }
       xml.etag();
       if (f.error() != QFile::NoError) {
-            QString s = tr("Write Instruments File failed: ") + f.errorString();
+            QString s = tr("Write Instruments File failed: %1").arg(f.errorString());
             QMessageBox::critical(this, tr("Write Instruments File"), s);
             }
       }
@@ -121,7 +129,7 @@ void InstrumentsDialog::on_loadButton_clicked()
           mscoreGlobalShare + "/templates",
          tr("MuseScore Instruments") + " (*.xml)",
          0,
-         preferences.nativeDialogs ? QFileDialog::Options() : QFileDialog::DontUseNativeDialog
+         preferences.getBool(PREF_UI_APP_USENATIVEDIALOGS) ? QFileDialog::Options() : QFileDialog::DontUseNativeDialog
          );
       if (fn.isEmpty())
             return;
@@ -204,30 +212,31 @@ void MuseScore::editInstrList()
             instrList->done(0);
             return;
             }
-      Score* rootScore = cs->rootScore();
-      instrList->genPartList(rootScore);
-      rootScore->startCmd();
-      rootScore->deselectAll();
+      instrList->init();
+      MasterScore* masterScore = cs->masterScore();
+      instrList->genPartList(masterScore);
+      masterScore->startCmd();
+      masterScore->deselectAll();
       int rv = instrList->exec();
 
       if (rv == 0) {
-            rootScore->endCmd();
+            masterScore->endCmd();
             return;
             }
-      ScoreView* cv = currentScoreView();
-      if (cv && cv->noteEntryMode()) {
-		cv->cmd(getAction("escape"));
+      ScoreView* csv = currentScoreView();
+      if (csv && csv->noteEntryMode()) {
+		csv->cmd(getAction("escape"));
             qApp->processEvents();
-            updateInputState(cv->score());
+            updateInputState(csv->score());
             }
-      rootScore->inputState().setTrack(-1);
+      masterScore->inputState().setTrack(-1);
 
       // keep the keylist of the first pitched staff to apply it to new ones
       KeyList tmpKeymap;
-      Staff* firstStaff = nullptr;
-      for (Staff* s : rootScore->staves()) {
+      Staff* firstStaff = 0;
+      for (Staff* s : masterScore->staves()) {
             KeyList* km = s->keyList();
-            if (!s->isDrumStaff()) {
+            if (!s->isDrumStaff(Fraction(0,1))) {     // TODO
                   tmpKeymap.insert(km->begin(), km->end());
                   firstStaff = s;
                   break;
@@ -235,7 +244,7 @@ void MuseScore::editInstrList()
             }
       Key normalizedC = Key::C;
       // normalize the keyevents to concert pitch if necessary
-      if (firstStaff && !rootScore->styleB(StyleIdx::concertPitch) && firstStaff->part()->instrument()->transpose().chromatic ) {
+      if (firstStaff && !masterScore->styleB(Sid::concertPitch) && firstStaff->part()->instrument()->transpose().chromatic ) {
             int interval = firstStaff->part()->instrument()->transpose().chromatic;
             normalizedC = transposeKey(normalizedC, interval);
             for (auto i = tmpKeymap.begin(); i != tmpKeymap.end(); ++i) {
@@ -277,72 +286,72 @@ void MuseScore::editInstrList()
             int rstaff = 0;
             PartListItem* pli = static_cast<PartListItem*>(item);
             if (pli->op == ListItemOp::I_DELETE)
-                  rootScore->cmdRemovePart(pli->part);
+                  masterScore->cmdRemovePart(pli->part);
             else if (pli->op == ListItemOp::ADD) {
                   const InstrumentTemplate* t = ((PartListItem*)item)->it;
-                  part = new Part(rootScore);
+                  part = new Part(masterScore);
                   part->initFromInstrTemplate(t);
-                  rootScore->undo(new InsertPart(part, staffIdx));
+                  masterScore->undo(new InsertPart(part, staffIdx));
 
                   pli->part = part;
                   QList<Staff*> linked;
                   for (int cidx = 0; pli->child(cidx); ++cidx) {
                         StaffListItem* sli = static_cast<StaffListItem*>(pli->child(cidx));
-                        Staff* staff       = new Staff(rootScore);
+                        Staff* staff       = new Staff(masterScore);
                         staff->setPart(part);
                         sli->setStaff(staff);
 
                         staff->init(t, sli->staffType(), cidx);
                         staff->setDefaultClefType(sli->defaultClefType());
+                        if (staffIdx > 0)
+                              staff->setBarLineSpan(masterScore->staff(staffIdx - 1)->barLineSpan());
 
-                        rootScore->undoInsertStaff(staff, cidx);
+                        masterScore->undoInsertStaff(staff, cidx);
                         ++staffIdx;
 
                         Staff* linkedStaff = part->staves()->front();
                         if (sli->linked() && linkedStaff != staff) {
-                              cloneStaff(linkedStaff, staff);
+                              Excerpt::cloneStaff(linkedStaff, staff);
                               linked.append(staff);
                               }
                         }
-                  if (linked.size() == 0)
-                        part->staves()->front()->setBarLineSpan(part->nstaves());
 
                   //insert keysigs
-                  int sidx = rootScore->staffIdx(part);
+                  int sidx = masterScore->staffIdx(part);
                   int eidx = sidx + part->nstaves();
                   if (firstStaff)
-                        rootScore->adjustKeySigs(sidx, eidx, tmpKeymap);
+                        masterScore->adjustKeySigs(sidx, eidx, tmpKeymap);
                   }
             else {
                   part = pli->part;
                   if (part->show() != pli->visible())
-                        part->undoChangeProperty(P_ID::VISIBLE, pli->visible());
+                        part->undoChangeProperty(Pid::VISIBLE, pli->visible());
                   for (int cidx = 0; pli->child(cidx); ++cidx) {
                         StaffListItem* sli = static_cast<StaffListItem*>(pli->child(cidx));
                         if (sli->op() == ListItemOp::I_DELETE) {
-                              rootScore->systems()->clear();
                               Staff* staff = sli->staff();
                               int sidx = staff->idx();
-                              rootScore->cmdRemoveStaff(sidx);
+                              masterScore->cmdRemoveStaff(sidx);
                               }
                         else if (sli->op() == ListItemOp::ADD) {
-                              Staff* staff = new Staff(rootScore);
+                              Staff* staff = new Staff(masterScore);
                               staff->setPart(part);
                               staff->initFromStaffType(sli->staffType());
                               sli->setStaff(staff);
                               staff->setDefaultClefType(sli->defaultClefType());
+                              if (staffIdx > 0)
+                                    staff->setBarLineSpan(masterScore->staff(staffIdx - 1)->barLineSpan());
 
                               KeySigEvent ke;
                               if (part->staves()->empty())
                                     ke.setKey(Key::C);
                               else
-                                    ke = part->staff(0)->keySigEvent(0);
+                                    ke = part->staff(0)->keySigEvent(Fraction(0,1));
 
-                              staff->setKey(0, ke);
+                              staff->setKey(Fraction(0,1), ke);
 
                               Staff* linkedStaff = 0;
                               if (sli->linked()) {
-
                                     if (rstaff > 0)
                                           linkedStaff = part->staves()->front();
                                     else {
@@ -357,19 +366,19 @@ void MuseScore::editInstrList()
                               if (linkedStaff) {
                                     // do not create a link if linkedStaff will be removed,
                                     for (int k = 0; pli->child(k); ++k) {
-                                          StaffListItem* i = static_cast<StaffListItem*>(pli->child(k));
-                                          if (i->op() == ListItemOp::I_DELETE && i->staff() == linkedStaff) {
+                                          StaffListItem* li = static_cast<StaffListItem*>(pli->child(k));
+                                          if (li->op() == ListItemOp::I_DELETE && li->staff() == linkedStaff) {
                                                 linkedStaff = 0;
                                                 break;
                                                 }
                                           }
                                     }
-                              rootScore->undoInsertStaff(staff, rstaff, linkedStaff == 0);
+                              masterScore->undoInsertStaff(staff, rstaff, linkedStaff == 0);
                               if (linkedStaff)
-                                    cloneStaff(linkedStaff, staff);
+                                    Excerpt::cloneStaff(linkedStaff, staff);
                               else {
                                     if (firstStaff)
-                                          rootScore->adjustKeySigs(staffIdx, staffIdx+1, tmpKeymap);
+                                          masterScore->adjustKeySigs(staffIdx, staffIdx+1, tmpKeymap);
                                     }
                               ++staffIdx;
                               ++rstaff;
@@ -380,8 +389,8 @@ void MuseScore::editInstrList()
                               const StaffType* stfType = sli->staffType();
 
                               // use selected staff type
-                              if (stfType->name() != staff->staffType()->name())
-                                    rootScore->undo(new ChangeStaffType(staff, *stfType));
+                              if (stfType->name() != staff->staffType(Fraction(0,1))->name())
+                                    masterScore->undo(new ChangeStaffType(staff, *stfType));
                               }
                         else {
                               ++staffIdx;
@@ -412,7 +421,7 @@ void MuseScore::editInstrList()
       int idx2 = 0;
       bool sort = false;
       for (Staff* staff : dst) {
-            int idx = rootScore->staves().indexOf(staff);
+            int idx = masterScore->staves().indexOf(staff);
             if (idx == -1)
                   qDebug("staff in dialog(%p) not found in score", staff);
             else
@@ -423,17 +432,17 @@ void MuseScore::editInstrList()
             }
 
       if (sort)
-            rootScore->undo(new SortStaves(rootScore, dl));
+            masterScore->undo(new SortStaves(masterScore, dl));
 
       //
       // check for valid barLineSpan and bracketSpan
       // in all staves
       //
-      for (Score* s : rootScore->scoreList()) {
+      for (Score* s : masterScore->scoreList()) {
             int n = s->nstaves();
             int curSpan = 0;
-            for (int i = 0; i < n; ++i) {
-                  Staff* staff = s->staff(i);
+            for (int j = 0; j < n; ++j) {
+                  Staff* staff = s->staff(j);
                   int span = staff->barLineSpan();
                   int setSpan = -1;
 
@@ -443,12 +452,12 @@ void MuseScore::editInstrList()
                         if (span == 0) {
                               // no span; this staff must have been within a span
                               // update it to a span of 1
-                              setSpan = 1;
+                              setSpan = j;
                               }
-                        else if (span > (n - i)) {
+                        else if (span > (n - j)) {
                               // span too big; staves must have been removed
                               // reduce span to last staff
-                              setSpan = n - i;
+                              setSpan = n - j;
                               }
                         else if (span > 1 && staff->barLineTo() > 0) {
                               // TODO: check if span is still valid
@@ -469,7 +478,7 @@ void MuseScore::editInstrList()
                         // except for Mensurstrich (barLineTo<=0)
                         // for consistency with Barline::endEdit,
                         // don't special case 1-line staves
-                        s->undoChangeBarLineSpan(staff, 0, 0, (staff->lines() - 1) * 2);
+//TODO                        s->undoChangeBarLineSpan(staff, 0, 0, (staff->lines() - 1) * 2);
                         }
 
                   // update barline span if necessary
@@ -477,21 +486,19 @@ void MuseScore::editInstrList()
                         // this staff starts a span
                         curSpan = setSpan;
                         // calculate spanFrom and spanTo values
-                        int spanFrom = staff->lines() == 1 ? BARLINE_SPAN_1LINESTAFF_FROM : 0;
-                        int linesTo = rootScore->staff(i + setSpan - 1)->lines();
-                        int spanTo = linesTo == 1 ? BARLINE_SPAN_1LINESTAFF_TO : (linesTo - 1) * 2;
-                        s->undoChangeBarLineSpan(staff, setSpan, spanFrom, spanTo);
+//                        int spanFrom = staff->lines() == 1 ? BARLINE_SPAN_1LINESTAFF_FROM : 0;
+//                        int linesTo = masterScore->staff(i + setSpan - 1)->lines();
+//                        int spanTo = linesTo == 1 ? BARLINE_SPAN_1LINESTAFF_TO : (linesTo - 1) * 2;
+//TODO                         s->undoChangeBarLineSpan(staff, setSpan, spanFrom, spanTo);
                         }
 
                   // count off one from barline span
                   --curSpan;
 
                   // update brackets
-                  QList<BracketItem> brackets = staff->brackets();
-                  int nn = brackets.size();
-                  for (int ii = 0; ii < nn; ++ii) {
-                        if ((brackets[ii]._bracket != BracketType::NO_BRACKET) && (brackets[ii]._bracketSpan > (n - i)))
-                              s->undoChangeBracketSpan(staff, ii, n - i);
+                  for (BracketItem* bi : staff->brackets()) {
+                        if ((bi->bracketSpan() > (n - j)))
+                              bi->undoChangeProperty(Pid::BRACKET_SPAN, n - j);
                         }
                   }
             }
@@ -499,20 +506,37 @@ void MuseScore::editInstrList()
       //
       // there should be at least one measure
       //
-      if (rootScore->measures()->size() == 0)
-            rootScore->insertMeasure(Element::Type::MEASURE, 0, false);
+      if (masterScore->measures()->size() == 0)
+            masterScore->insertMeasure(ElementType::MEASURE, 0, false);
 
-      QList<Score*> toDelete;
-      for (Excerpt* excpt : rootScore->excerpts()) {
-            if (excpt->partScore()->staves().size() == 0)
-                  toDelete.append(excpt->partScore());
+      const QList<Excerpt*> excerpts(masterScore->excerpts()); // excerpts list may change in the loop below
+      for (Excerpt* excerpt : excerpts) {
+            QList<Staff*> sl       = excerpt->partScore()->staves();
+            QMultiMap<int, int> tr = excerpt->tracks();
+            if (sl.size() == 0)
+                  masterScore->undo(new RemoveExcerpt(excerpt));
+            else {
+                  for (Staff* s : sl) {
+                        const LinkedElements* sll = s->links();
+                        for (auto le : *sll) {
+                              Staff* ss = toStaff(le);
+                              if (ss->primaryStaff()) {
+                                    for (int j = s->idx() * VOICES; j < (s->idx() + 1) * VOICES; j++) {
+                                          int strack = tr.key(j, -1);
+                                          if (strack != -1 && ((strack & ~3) == ss->idx()))
+                                                break;
+                                          else if (strack != -1)
+                                                tr.insert(ss->idx() + strack % VOICES, tr.value(strack, -1));
+                                          }
+                                    }
+                              }
+                        }
+                  }
             }
-      for(Score* s: toDelete)
-            rootScore->undo(new RemoveExcerpt(s));
 
-      rootScore->setLayoutAll(true);
-      rootScore->endCmd();
-      rootScore->rebuildMidiMapping();
+      masterScore->setLayoutAll();
+      masterScore->endCmd();
+      masterScore->rebuildAndUpdateExpressive(MuseScore::synthesizer("Fluid"));
       seq->initInstruments();
       }
 
