@@ -1,7 +1,6 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: palette.h 5395 2012-02-28 18:09:57Z wschweer $
 //
 //  Copyright (C) 2002-2011 Werner Schweer and others
 //
@@ -29,7 +28,7 @@ namespace Ms {
 
 class Element;
 class Sym;
-class Xml;
+class XmlWriter;
 class XmlReader;
 class Palette;
 
@@ -43,6 +42,7 @@ struct PaletteCell {
       Element* element { 0 };
       QString name;           // used for tool tip
       QString tag;
+
       bool drawStaff { false };
       double x       { 0.0   };
       double y       { 0.0   };
@@ -90,6 +90,9 @@ class PaletteScrollArea : public QScrollArea {
 
       virtual void resizeEvent(QResizeEvent*);
 
+   protected:
+      virtual void keyPressEvent(QKeyEvent* event) override;
+
    public:
       PaletteScrollArea(Palette* w, QWidget* parent = 0);
       bool restrictHeight() const { return _restrictHeight; }
@@ -105,13 +108,14 @@ class Palette : public QWidget {
 
       QString _name;
       QList<PaletteCell*> cells;
+      QList<PaletteCell*> dragCells;  // used for filter & backup
 
-      int hgrid, vgrid;
+      int hgrid;
+      int vgrid;
       int currentIdx;
       int dragIdx;
       int selectedIdx;
       QPoint dragStartPosition;
-      int dragSrcIdx;
 
       qreal extraMag;
       bool _drawGrid;
@@ -119,42 +123,48 @@ class Palette : public QWidget {
       bool _disableDoubleClick { false };
       bool _readOnly;
       bool _systemPalette;
-      qreal _yOffset;         // in spatium units of "gscore"
+      qreal _yOffset;                // in spatium units of "gscore"
+      bool filterActive { false };   // bool if filter is active
 
       bool _moreElements;
+      bool _showContextMenu { true };
 
-      void redraw(const QRect&);
-      virtual void paintEvent(QPaintEvent*);
-      virtual void mousePressEvent(QMouseEvent*);
-      virtual void mouseDoubleClickEvent(QMouseEvent*);
-      virtual void mouseMoveEvent(QMouseEvent*);
-      virtual void leaveEvent(QEvent*);
-      virtual bool event(QEvent*);
-      virtual void resizeEvent(QResizeEvent*);
+      virtual void paintEvent(QPaintEvent*) override;
+      virtual void mousePressEvent(QMouseEvent*) override;
+      virtual void mouseDoubleClickEvent(QMouseEvent*) override;
+      virtual void mouseMoveEvent(QMouseEvent*) override;
+      virtual void leaveEvent(QEvent*) override;
+      virtual bool event(QEvent*) override;
+      virtual void resizeEvent(QResizeEvent*) override;
+      void applyPaletteElement(PaletteCell* cell, Qt::KeyboardModifiers modifiers = 0);
 
-      virtual void dragEnterEvent(QDragEnterEvent*);
-      virtual void dragMoveEvent(QDragMoveEvent*);
-      virtual void dropEvent(QDropEvent*);
-      virtual void contextMenuEvent(QContextMenuEvent*);
+      virtual void dragEnterEvent(QDragEnterEvent*) override;
+      virtual void dragMoveEvent(QDragMoveEvent*) override;
+      virtual void dropEvent(QDropEvent*) override;
+      virtual void contextMenuEvent(QContextMenuEvent*) override;
+
+      int idx2(const QPoint&) const;
+      QRect idxRect(int) const;
+
+      const QList<PaletteCell*>* ccp() const { return filterActive ? &dragCells : &cells; }
+      QPixmap pixmap(int cellIdx) const;
 
-      int idx(const QPoint&) const;
-      QRect idxRect(int);
-      void layoutCell(PaletteCell*);
 
    private slots:
       void actionToggled(bool val);
 
    signals:
-      void startDragElement(Element*);
       void boxClicked(int);
       void changed();
-      void moreButtonClicked();
       void displayMore(const QString& paletteName);
 
    public:
       Palette(QWidget* parent = 0);
       virtual ~Palette();
 
+      void nextPaletteElement();
+      void prevPaletteElement();
+      void applyPaletteElement();
       PaletteCell* append(Element*, const QString& name, QString tag = QString(),
          qreal mag = 1.0);
       PaletteCell* add(int idx, Element*, const QString& name,
@@ -168,7 +178,7 @@ class Palette : public QWidget {
       bool read(const QString& path);
       void write(const QString& path);
       void read(XmlReader&);
-      void write(Xml&) const;
+      void write(XmlWriter&) const;
       bool read(QFile*);
       void clear();
       void setSelectable(bool val)   { _selectable = val;  }
@@ -188,19 +198,25 @@ class Palette : public QWidget {
       qreal yOffset() const          { return _yOffset;        }
       int columns() const            { return width() / hgrid; }
       int rows() const;
-      int size() const               { return cells.size(); }
-      void setCellReadOnly(int c, bool v) { cells[c]->readOnly = v; }
+      int size() const               { return filterActive ? dragCells.size() : cells.size(); }
+      PaletteCell* cellAt(int index) const { return ccp()->value(index); }
+      void setCellReadOnly(int c, bool v)  { cells[c]->readOnly = v;   }
       QString name() const           { return _name;        }
       void setName(const QString& s) { _name = s;           }
       int gridWidth() const          { return hgrid;        }
       int gridHeight() const         { return vgrid;        }
       bool moreElements() const      { return _moreElements; }
       void setMoreElements(bool val);
+      bool filter(const QString& text);
+      void setShowContextMenu(bool val) { _showContextMenu = val; }
 
-      QString getCellName(int idx) const { return cells[idx]->name; }
-
+      int getCurrentIdx() { return currentIdx; }
+      void setCurrentIdx(int i) { currentIdx = i; }
+      bool isFilterActive() { return filterActive == true; }
+      QList<PaletteCell*> getDragCells() { return dragCells; }
       virtual int heightForWidth(int) const;
       virtual QSize sizeHint() const;
+      int idx(const QPoint&) const;
       };
 
 
