@@ -1,7 +1,6 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: seq.h 5660 2012-05-22 14:17:39Z wschweer $
 //
 //  Copyright (C) 2002-2009 Werner Schweer and others
 //
@@ -21,6 +20,7 @@
 #ifndef __SEQ_H__
 #define __SEQ_H__
 
+#include "libmscore/rendermidi.h"
 #include "libmscore/sequencer.h"
 #include "libmscore/fraction.h"
 #include "synthesizer/event.h"
@@ -33,13 +33,14 @@ class QTimer;
 namespace Ms {
 
 class Note;
+class MasterScore;
 class Score;
 class Painter;
 class Measure;
 class Fraction;
 class Driver;
 class Part;
-struct Channel;
+class Channel;
 class ScoreView;
 class MasterSynthesizer;
 class Segment;
@@ -105,7 +106,7 @@ class Seq : public QObject, public Sequencer {
 
       mutable QMutex mutex;
 
-      Score* cs;
+      MasterScore* cs;
       ScoreView* cv;
       bool running;                       // true if sequencer is available
       Transport state;                    // STOP, PLAY, STARTING=3
@@ -132,7 +133,14 @@ class Seq : public QObject, public Sequencer {
       double meterPeakValue[2];
       int peakTimer[2];
 
-      EventMap events;                    // playlist for playback mode (pre-rendered)
+      EventMap events;                    // playlist for playback mode
+      EventMap::const_iterator eventsEnd;
+      EventMap renderEvents;              // event list that is rendered in background
+      RangeMap renderEventsStatus;
+      MidiRenderer midi;
+      QFuture<void> midiRenderFuture;
+      bool allowBackgroundRendering = false; // should be set to true only when playing, so no
+                                             // score changes are possible.
       EventMap countInEvents;             // playlist of any metronome countin clicks
       QQueue<NPlayEvent> _liveEventQueue; // playlist for score editing and note entry (rendered live)
 
@@ -157,7 +165,8 @@ class Seq : public QObject, public Sequencer {
       QTimer* heartBeatTimer;
       QTimer* noteTimer;
 
-      void collectMeasureEvents(Measure*, int staffIdx);
+      void renderChunk(const MidiRenderer::Chunk&, EventMap*);
+      void updateEventsEnd();
 
       void setPos(int);
       void playEvent(const NPlayEvent&, unsigned framePos);
@@ -168,6 +177,8 @@ class Seq : public QObject, public Sequencer {
       void updateSynthesizerState(int tick1, int tick2);
       void addCountInClicks();
 
+      int getPlayStartUtick();
+
       inline QQueue<NPlayEvent>* liveEventQueue() { return &_liveEventQueue; }
 
    private slots:
@@ -207,7 +218,8 @@ class Seq : public QObject, public Sequencer {
       void prevMeasure();
       void prevChord();
 
-      void collectEvents();
+      void collectEvents(int utick);
+      void ensureBufferAsync(int utick);
       void guiStop();
       void stopWait();
       void setLoopIn();
@@ -229,7 +241,7 @@ class Seq : public QObject, public Sequencer {
       void setController(int, int, int);
       virtual void sendEvent(const NPlayEvent&);
       void setScoreView(ScoreView*);
-      Score* score() const   { return cs; }
+      MasterScore* score() const   { return cs; }
       ScoreView* viewer() const { return cv; }
       void initInstruments(bool realTime = false);
 
