@@ -52,7 +52,7 @@ UpdateChecker::UpdateChecker(QObject* parent)
 
 void UpdateChecker::onRequestFinished(QNetworkReply* reply)
       {
-      if(reply->error() != QNetworkReply::NoError){
+      if (reply->error() != QNetworkReply::NoError) {
             qDebug("Error while checking update [%s]", reply->errorString().toLatin1().constData());
             return;
             }
@@ -122,14 +122,14 @@ QString UpdateChecker::parseText(QXmlStreamReader& reader)
       {
       QString result;
       reader.readNext();
-      if(reader.tokenType() == QXmlStreamReader::Characters)
+      if (reader.tokenType() == QXmlStreamReader::Characters)
             result = reader.text().toString();
       return result;
       }
 
 bool UpdateChecker::getUpdatePrefValue()
       {
-      return preferences.checkUpdateStartup;
+      return preferences.getBool(PREF_UI_APP_STARTUP_CHECKUPDATE);
       }
 
 QString UpdateChecker::getUpdatePrefString()
@@ -146,21 +146,25 @@ void UpdateChecker::check(QString currentVersion, bool m)
 #if defined(Q_OS_MAC)
       os = "mac";
 #endif
-      if(qApp->applicationName() == "MuseScore2") { //avoid nightly cymbals
-            if(MuseScore::unstable())
-                  release = "pre";
-            else
-                  release = "stable";
+      bool check = true;
+      if (MuseScore::unstable()) {
+            release = "nightly";
+            QString buildNumber = QString("%1").arg(BUILD_NUMBER);
+            if (!buildNumber.isEmpty())
+                  _currentVersion = QString("%1.%2").arg(currentVersion).arg(BUILD_NUMBER);
+            else {
+                  _currentVersion = currentVersion;
+                  check = false;
+                  }
             }
       else {
-            release = "nightly";
+            release = "stable";
+             _currentVersion =  currentVersion;
             }
       if (MScore::debugMode)
             qDebug("release type: %s", release.toLatin1().constData());
-      if(!os.isEmpty() && !release.isEmpty() && release != "nightly"){
-            _currentVersion =  currentVersion;
+      if (!os.isEmpty() && !release.isEmpty() && check)
             manager->get(QNetworkRequest(QUrl("http://update.musescore.org/update_" + os +"_" + release +".xml")));
-            }
       }
 
 UpdateCheckerBase::UpdateCheckerBase(QObject* parent)
@@ -177,7 +181,8 @@ bool UpdateCheckerBase::hasToCheck()
       {
       if(!getUpdatePrefValue())
             return false;
-
+//disable embedded updating for both stable/unstable Mac builds and stable Win builds
+#if !defined(Q_OS_MAC) && (!defined(Q_OS_WIN) || defined(MSCORE_UNSTABLE))
       QSettings s;
       s.beginGroup("Update");
       QDateTime now = QDateTime::currentDateTime();
@@ -189,6 +194,9 @@ bool UpdateCheckerBase::hasToCheck()
             }
       s.endGroup();
       return now == lastUpdate || now > lastUpdate.addSecs(3600 * defaultPeriod()) ;
+#else
+      return true;
+#endif
       }
 
 ExtensionsUpdateChecker::ExtensionsUpdateChecker(QObject* parent)
@@ -225,7 +233,7 @@ void ExtensionsUpdateChecker::check()
                   if (compareVersion(installedVersion, version)) {
                         QMessageBox msgBox;
                         msgBox.setWindowTitle(tr("Extension Updates Available"));
-                        msgBox.setText(tr("One or more installed extensions have updates available in Help / Resource Manager..."));
+                        msgBox.setText(tr("One or more installed extensions have updates available in Help > Resource Manager…"));
                         msgBox.setTextFormat(Qt::RichText);
                         msgBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
                         msgBox.setDefaultButton(QMessageBox::Ok);
@@ -256,7 +264,7 @@ void ExtensionsUpdateChecker::check()
 
 bool ExtensionsUpdateChecker::getUpdatePrefValue()
       {
-      return preferences.checkExtensionsUpdateStartup;
+      return preferences.getBool(PREF_UI_APP_STARTUP_CHECK_EXTENSIONS_UPDATE);
       }
 
 QString ExtensionsUpdateChecker::getUpdatePrefString()
