@@ -1,7 +1,6 @@
 //=============================================================================
 //  MusE
 //  Linux Music Editor
-//  $Id:$
 //
 //  Copyright (C) 2002-2010 by Werner Schweer and others
 //
@@ -18,6 +17,9 @@
 //  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //=============================================================================
 
+#ifndef MUSESCORE_CONFIG_H
+#define MUSESCORE_CONFIG_H
+
 #define USE_ALSA
 #define USE_JACK
 /* #undef USE_PORTAUDIO */
@@ -36,7 +38,7 @@
 #define VERSION           "2.0.0"
 
 #define AEOLUS
-// #define OMR
+/* #undef OMR */
 /* #undef OCR */
 #define OSC
 /* #undef OPENGL */
@@ -47,7 +49,4 @@
 #define USE_BSP         true
 #define SCRIPT_INTERFACE true
 
-#if QT_VERSION >= 0x040800
-//#define USE_GLYPHS  1
-#endif
-
+#endif /* MUSESCORE_CONFIG_H */
