@@ -2,7 +2,6 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2
@@ -50,7 +49,7 @@ void TestSfzEnvelopes::initTestCase()
       initMTest();
       synth = new Zerberus();
       synth->init(samplerate);
-      Ms::preferences.mySoundfontsPath += ";" + root;
+      preferences.setPreference(PREF_APP_PATHS_MYSOUNDFONTS, root);
       synth->loadInstrument("envelopesTest.sfz");
       }
 
@@ -60,7 +59,7 @@ void TestSfzEnvelopes::initTestCase()
 
 void TestSfzEnvelopes::testEnvelopesParsing()
       {
-      QCOMPARE(synth->instrument(0)->zones().size(), (unsigned long) 1);
+      QCOMPARE(synth->instrument(0)->zones().size(), (size_t) 1);
       QCOMPARE(synth->instrument(0)->zones().front()->ampegDelay, 0.01f * 1000.0f);
       QCOMPARE(synth->instrument(0)->zones().front()->ampegStart, 20.0f / 100.0f); // 20 percent
       QCOMPARE(synth->instrument(0)->zones().front()->ampegAttack, 0.01f * 1000.0f);
